/*******************************************************************************
 * Copyright (C) 2015 Steinbeis GmbH & Co. KG.
 * All rights reserved. 
 *
 * This program and the accompanying materials are made 
 * available under the terms of the Track+ License v2.0
 * which is available at http://www.trackplus.com/tpl-v20.html
 *
 * $Id:$
 *******************************************************************************/
package com.trackplus.track.ws.samples;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.rmi.RemoteException;

import com.trackplus.track.ws.beans.WSAttachmentBean;
import com.trackplus.track.ws.beans.WSEditItemContextBean;
import com.trackplus.track.ws.beans.WSFieldsConfigBean;
import com.trackplus.track.ws.beans.WSItemBean;
import com.trackplus.track.ws.beans.WSLabelValueBean;
import com.trackplus.track.ws.beans.WSSortedOptionBean;
import com.trackplus.track.ws.tcl.TCLFacade;
import com.trackplus.track.ws.tcl.TCLFacadeException;

/**
 * <p>
 * This class demonstrates how to use the TCl library to work with attachments.
 * This example uses the {@link LoginSample} code, where you need to insert your
 * servers URL, user name, and password.
 * </p>
 * <p>
 * This example is a Java console application. You can run it directly by
 * calling its <code>main()</code> method.
 * </p>
 * 
 */
public class AttachmentSample extends LoginSample {
	public AttachmentSample() {
		super();
	}

	public AttachmentSample(TCLFacade connector) {
		super(connector);
	}

	public WSAttachmentBean addAttachment(String itemId, String attachName, String attachDescription, File file) {
		System.out.println("\n> AttachmentSample.addAttachemt()");
		WSAttachmentBean attach = new WSAttachmentBean();
		attach.setAttachName(attachName);
		attach.setDescription(attachDescription);
		attach.setContentType("text/html");
		attach.setItemKey(itemId);
		attach.setSize(file.length() + "");
		attach.setFileName(file.getPath());
		WSAttachmentBean attachResult = null;
		try {
			attachResult = connector.saveAttachment(attach);
			System.out.println("   < Successfully add the attachments:" + attachResult.getAttachName() + " on item:" + itemId);
		} catch (RemoteException e) {
			System.out.println("   < Can't add attachments");
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("   < Can't add attachments");
			handleTCLFacadeException(e);
		}
		System.out.println("   < AttachmentSample.addAttachemt()");
		return attachResult;
	}

	public void printAttachments(WSAttachmentBean[] attachments) {
		if (attachments == null || attachments.length == 0) {
			System.out.println("   < No attachments\n\n");
		} else {
			System.out.println("\n" + attachments.length + " attachments found:");
			for (int i = 0; i < attachments.length; i++) {
				System.out.println("\tID=" + attachments[i].getAttachKey());
				System.out.println("\tDate=" + attachments[i].getCreateDate());
				System.out.println("\tOriginator=" + attachments[i].getOriginator().getPersonID() + ":" + attachments[i].getOriginator().getFullName());
				System.out.println("\tName=" + attachments[i].getAttachName());
				System.out.println("\tDescription=" + attachments[i].getDescription());
				System.out.println();
			}
			System.out.println("\n");
		}
	}

	public WSAttachmentBean[] getItemAttachments(String itemId) {
		WSAttachmentBean[] attachments = null;
		try {
			WSItemBean item = connector.getItem(itemId);
			attachments = item.getAttachments();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		return attachments;
	}

	public void addAttachmentInputStream(String itemId, String attachName, String attachDescription, File file) {
		System.out.println("\n> AttachmentSample.addAttachmentInputStream()");
		WSAttachmentBean attach = new WSAttachmentBean();
		attach.setAttachName(attachName);
		attach.setDescription(attachDescription);
		attach.setContentType("img/jpeg");
		attach.setItemKey(itemId);
		attach.setSize(file.length() + "");
		attach.setFileName(file.getPath());
		try {
			WSAttachmentBean attachResult = connector.saveAttachment(attach, new MyInputStreamProvider(file));
			System.out.println("   < Successfully added the attachment as stream :" + attachResult.getAttachName() + " on item:" + itemId);
		} catch (RemoteException e) {
			System.out.println("   < Can't add attachments");
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("   < Can't add attachments");
			handleTCLFacadeException(e);
		}
		System.out.println("< AttachmentSample.addAttachmentInputStream()");
	}

	public void deleteAttachemnt(WSAttachmentBean attach) {
		System.out.println("\n> AttachmentSample.deleteAttachemnt()");
		try {
			connector.deleteAttachment(attach);
			System.out.println("   < Successfully added the attachment as stream :" + attach.getAttachName() + " on item:" + attach.getItemKey());
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		System.out.println("< AttachmentSample.deleteAttachemnt()");
	}

	public void downloadAttachment(WSAttachmentBean attach) {
		System.out.println("\n> AttachmentSample.downloadAttachment()");
		InputStream is = null;
		try {
			is = connector.downloadAttachment(attach);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		File downloadDir = new File("etc" + File.separator + "download");
		downloadDir.mkdirs();
		String fileName = downloadDir.getAbsolutePath() + File.separator + attach.getAttachName();
		try {
			writeToFile(is, fileName);
			System.out.println("Successfully download the  attachment:" + attach.getAttachName() + " in " + fileName);
		} catch (IOException e) {
			e.printStackTrace();
		}
		// File downloaded=new File(fileName);
		// downloaded.delete();
		System.out.println("< AttachmentSample.downloadAttachment()");
	}

	public static void writeToFile(InputStream is, String fileName) throws IOException {
		BufferedOutputStream fOut = null;
		try {
			fOut = new BufferedOutputStream(new FileOutputStream(fileName));
			byte[] buffer = new byte[32 * 1024];
			int bytesRead = 0;
			while ((bytesRead = is.read(buffer)) != -1) {
				fOut.write(buffer, 0, bytesRead);
			}
			fOut.flush();
		}finally {
			fOut.close();
			is.close();
		}
	}

	public String createItem() {
		// to create an item we need an project and issueType
		String projectID = null;
		String issueTypeID = null;
		// getting the project for the user
		try {
			WSLabelValueBean[] projects = connector.getProjects();
			if (projects == null || projects.length == 0) {
				System.out.println("No projects found for user: " + uName);
				return null;
			}
			projectID = projects[0].getValue();
			WSLabelValueBean[] issueTypes = connector.getIssueTypes(projectID);
			if (issueTypes == null || issueTypes.length == 0) {
				System.out.println("No issueTypes found for user: " + uName);
				return null;
			}
			issueTypeID = issueTypes[0].getValue();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Error obtaining the project and item type");
			handleTCLFacadeException(e);
			return null;
		}
		WSEditItemContextBean editItemCtx = null;
		try {
			editItemCtx = connector.getNewItemContext(projectID, issueTypeID);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Error obtaining the newItemContext");
			handleTCLFacadeException(e);
		}
		if (editItemCtx != null) {
			WSItemBean item = editItemCtx.getItem();

			// the fieldsConfig contains settings(label, tooltip, readOnly) and
			// options only for fields
			// present on the form configured for NEW item in given project and
			// issueType
			// and if we change a field that is not present on the form will do
			// not have effect.
			WSFieldsConfigBean cfg = editItemCtx.getFieldsConfig();

			// set some values
			item.setSynopsis("WebService item");
			item.setDescription("This item was creared from webservice");
			// set the priority( assume the priority is present on the form)
			// the priorities length must be >=1 because include the default
			// value
			WSSortedOptionBean[] priorities = cfg.getPriorityOptions();
			String priorityID = priorities[priorities.length - 1].getObjectID();
			item.setPriorityID(priorityID);
			// set the comment
			item.setNewComment("Comment from web-service");

			// save the item
			String itemID = null;
			try {
				itemID = connector.saveItem(item);
				System.out.println("Item saved successfully: The new itemID is:" + itemID);
			} catch (RemoteException e) {
				System.out.println("Failed to save the item");
				e.printStackTrace();
			} catch (TCLFacadeException e) {
				System.out.println("Failed to save the item");
				handleTCLFacadeException(e);
			}
			return itemID;
		}
		return null;
	}

	class MyInputStreamProvider implements TCLFacade.InputStreamProvider {
		private File file;

		public MyInputStreamProvider(File file) {
			this.file = file;
		}

		public InputStream createInputStream() {
			try {
				return new FileInputStream(file);
			} catch (FileNotFoundException e) { 
				e.printStackTrace();
			}
			return null;
		}
	}

	public static void main(String[] args) {
		if (args.length >= 2) {
		uName = (args[0]);
		pField = args[1];
		}
		if (args.length > 2) {
			url = args[3];
		}

		AttachmentSample attachmentSample = new AttachmentSample();
		attachmentSample.login();

		String itemID = attachmentSample.createItem();

		if (itemID != null) {
			// addAttachent
			String attachName = "dummyHtml.html";
			String attachDescription = "some attach description";
			File file = new File("etc" + File.separator + "dummyHtml.html");
			attachmentSample.addAttachment(itemID, attachName, attachDescription, file);

			// see the attachment
			WSAttachmentBean[] attachments = attachmentSample.getItemAttachments(itemID);
			attachmentSample.printAttachments(attachments);

			// add an image attachment
			String attachName2 = "Hovering.jpg";
			String attachDescription2 = "some attach description2";
			File file2 = new File("etc" + File.separator + "Hovering.jpg");
			attachmentSample.addAttachmentInputStream(itemID, attachName2, attachDescription2, file2);

			// see the attachment
			attachments = attachmentSample.getItemAttachments(itemID);
			attachmentSample.printAttachments(attachments);

			// download attachment
			// see etc/download directory
			attachmentSample.downloadAttachment(attachments[0]);

			// delete the attachment
			attachmentSample.deleteAttachemnt(attachments[0]);

			// see the attachments
			attachments = attachmentSample.getItemAttachments(itemID);
			attachmentSample.printAttachments(attachments);
		} else {
			System.out.println("Can't create item");
		}
	}
}
