/*******************************************************************************
 * Copyright (C) 2015 Steinbeis GmbH & Co. KG.
 * All rights reserved. 
 *
 * This program and the accompanying materials are made 
 * available under the terms of the Track+ License v2.0
 * which is available at http://www.trackplus.com/tpl-v20.html
 *
 * $Id:$
 *******************************************************************************/

package com.trackplus.track.ws.samples;

import java.rmi.RemoteException;

import com.trackplus.track.ws.beans.WSFormBean;
import com.trackplus.track.ws.beans.WSFormFieldBean;
import com.trackplus.track.ws.beans.WSFormPanelBean;
import com.trackplus.track.ws.beans.WSFormTabBean;
import com.trackplus.track.ws.beans.WSLabelValueBean;
import com.trackplus.track.ws.tcl.TCLFacade;
import com.trackplus.track.ws.tcl.TCLFacadeException;

/**
 * <p>
 * In Track+ you can define any number of forms (input masks) and assign them to
 * actions like edit, change status, and so on, depending on project and issue
 * type.
 * </p>
 * <p>
 * This class demonstrates how to access the form definition information via the
 * web service interface.
 * </p>
 * <p>
 * The class uses the {@link LoginSample} class to perform the login. You need
 * to set the URL and the user name and password in that class before you
 * compile and run this class here.
 * </p>
 * <p>
 * This example is a Java console application. You can run it directly by
 * calling its <code>main()</code> method.
 * </p>
 */

public class FormSample extends LoginSample {
	public FormSample() {
		super();
	}

	public FormSample(TCLFacade connector) {
		super(connector);
	}

	public void getEditForm() {
		System.out.println("\n->FormSample.getEditForm()");
		String projectID = null;
		String issueTypeID = null;
		// getting the project for the user
		try {
			WSLabelValueBean[] projects = connector.getProjects();
			if (projects == null || projects.length == 0) {
				System.out.println("No projects found for user:" + uName);
				return;
			}
			projectID = projects[0].getValue();
			WSLabelValueBean[] issueTypes = connector.getIssueTypes(projectID);
			if (issueTypes == null || issueTypes.length == 0) {
				System.out.println("No issueTypes found for user:" + uName);
				return;
			}
			issueTypeID = issueTypes[0].getValue();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Error obtaining the project and issue type");
			handleTCLFacadeException(e);
			return;
		}
		try {
			// if projectID==null and issueTypeID==null we obtain the default
			// edit form
			WSFormBean formEdit = connector.getEditItemForm(projectID, issueTypeID);
			System.out.println("Successfully obtained the form for project=" + projectID + " and issueTypes=" + issueTypeID);
			printForm(formEdit);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Error obtaining the form");
			handleTCLFacadeException(e);
		}
		System.out.println("<-FormSample.getEditForm()\n");
	}

	public static void printForm(WSFormBean formBean) {
		WSFormTabBean[] tabs = formBean.getTabs();
		for (int i = 0; i < tabs.length; i++) {
			WSFormTabBean tab = tabs[i];
			printFormTab(tab);
		}
	}

	private static void printFormTab(WSFormTabBean tab) {
		System.out.println("_________________");
		System.out.println("|TAB: " + tab.getLabel() + "|");
		System.out.println("_________________");
		System.out.println("");
		WSFormPanelBean[] panels = tab.getPanels();
		for (int i = 0; i < panels.length; i++) {
			WSFormPanelBean panel = panels[i];
			printFormPanel(panel);
		}
		System.out.println("\n");
	}

	private static void printFormPanel(WSFormPanelBean panel) {
		System.out.println("************************************************");
		System.out.println("PANEL: Rows=" + panel.getRows() + "\tCols=" + panel.getCols());
		WSFormFieldBean[] fields = panel.getFields();
		for (int i = 0; i < fields.length; i++) {
			WSFormFieldBean fieldBean = fields[i];
			System.out.println("FORM_FIELD: FieldID=" + fieldBean.getFieldID() + " POS(" + fieldBean.getRowIndex() + "," + fieldBean.getColIndex() + ") SPAN("
					+ fieldBean.getRowSpan() + "," + fieldBean.getColSpan() + ")" + " Label_ALIGN(" + getHorizontalAlign(fieldBean.getLabelHAlign()) + ","
					+ getVerticalAlign(fieldBean.getLabelVAlign()) + ")" + " Value_ALIGN(" + getHorizontalAlign(fieldBean.getValueHAlign()) + ","
					+ getVerticalAlign(fieldBean.getValueVAlign()) + ")");
		}
		System.out.println("************************************************\n");
	}

	public static String getVerticalAlign(String valignStr) {
		int valign = TCLFacade.TOP;
		try {
			valign = Integer.parseInt(valignStr);
		} catch (Exception ex) {
			valign = TCLFacade.TOP;
		}
		return getVerticalAlign(valign);
	}

	public static String getVerticalAlign(int valign) {
		switch (valign) {
		case TCLFacade.MIDDLE: {
			return "middle";
		}
		case TCLFacade.TOP: {
			return "top";
		}
		case TCLFacade.BOTTOM: {
			return "bottom";
		}
		}
		return "top";
	}

	public static String getHorizontalAlign(String halignStr) {
		int halign = TCLFacade.LEFT;
		try {
			halign = Integer.parseInt(halignStr);
		} catch (Exception ex) {
			halign = TCLFacade.LEFT;
		}
		return getHorizontalAlign(halign);
	}

	public static String getHorizontalAlign(int halign) {
		switch (halign) {
		case TCLFacade.CENTER: {
			return "center";
		}
		case TCLFacade.LEFT: {
			return "left";
		}
		case TCLFacade.RIGHT: {
			return "right";
		}
		}
		return "left";
	}

	public static void main(String[] args) {

		if (args.length >= 2) {
			uName = (args[0]);
			pField = args[1];
		}
		if (args.length > 2) {
			url = args[3];
		}
		FormSample formSample = new FormSample();
		formSample.login();

		formSample.getEditForm();

	}

}
