/*******************************************************************************
 * Copyright (C) 2015 Steinbeis GmbH & Co. KG.
 * All rights reserved. 
 *
 * This program and the accompanying materials are made 
 * available under the terms of the Track+ License v2.0
 * which is available at http://www.trackplus.com/tpl-v20.html
 *
 * $Id:$
 *******************************************************************************/
package com.trackplus.track.ws.samples;

import java.rmi.RemoteException;

import com.trackplus.track.ws.beans.WSGroupBean;
import com.trackplus.track.ws.beans.WSGroupEditBean;
import com.trackplus.track.ws.beans.WSPersonBean;
import com.trackplus.track.ws.tcl.TCLFacade;
import com.trackplus.track.ws.tcl.TCLFacadeException;

public class GroupSample extends LoginSample {
	public GroupSample() {
		super();
	}

	public GroupSample(TCLFacade connector) {
		super(connector);
	}

	public WSGroupBean[] getAllGroups() {
		try {
			WSGroupBean[] allGroups = connector.getAllGroups();
			return allGroups;
		} catch (RemoteException e) {
			e.printStackTrace();
			return null;
		} catch (TCLFacadeException e) {
			e.printStackTrace();
			return null;
		}
	}

	public String addGroup(String groupName) throws RemoteException, TCLFacadeException {
		WSGroupEditBean newGroup = new WSGroupEditBean();
		newGroup.setName(groupName);
		newGroup.setInheritAuthor(true);
		newGroup.setInheritManager(false);
		newGroup.setInheritResponsible(true);
		String groupID = connector.saveGroup(newGroup);
		return groupID;
	}

	private void printGroups(WSGroupBean[] allGroups) {
		if (allGroups != null) {
			WSGroupBean group;
			for (int i = 0; i < allGroups.length; i++) {
				group = allGroups[i];
				System.out.println(i + " {name=" + group.getName() + " ;ID=" + group.getObjectID() + "}");
			}
		} else {
			System.out.println("No groups");
		}
	}

	public WSGroupEditBean getGroupEdit(String groupID) {
		try {
			WSGroupEditBean groupEditBean = connector.getEditGroup(groupID);
			return groupEditBean;
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return null;
	}

	public void deleteGroup(String groupID) {
		try {
			connector.deleteGroup(groupID);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void addPersonsToGroup(String groupID, String[] personIDs) {
		try {
			connector.addPersonsToGroup(groupID, personIDs);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void removePersonsFromGroup(String groupID, String[] personIDs) {
		try {
			connector.removePersonsFromGroup(groupID, personIDs);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public WSPersonBean[] getPersonsForGroup(String groupID) {
		try {
			WSPersonBean[] persons = connector.getPersonsForGroup(groupID);
			return persons;
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		} catch (TCLFacadeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
	}

	private void printEditGroup(WSGroupEditBean groupEditBean) {
		System.out.println("WSGroupEditBean:[");
		System.out.println("objectID=" + groupEditBean.getObjectID());
		System.out.println("name=" + groupEditBean.getName());
//		System.out.println("originator=" + groupEditBean.getInheritAuthor());
//		System.out.println("manager=" + groupEditBean.getInheritManager());
//		System.out.println("responsible=" + groupEditBean.getInheritResponsible());
		System.out.println("]");

	}

	public static void main(String[] args) {
		GroupSample groupSample = new GroupSample();
		// login
		groupSample.login(uName, pField);
		WSGroupBean[] allGroups = groupSample.getAllGroups();
		System.out.println("Initial groups:");
		groupSample.printGroups(allGroups);
		System.out.println("\n");
		try {
			String groupID = groupSample.addGroup("group1");
			System.out.println("Group saved with id:" + groupID + "\n");

			WSGroupEditBean groupEditBean = groupSample.getGroupEdit(groupID);
			System.out.println("new group added:");
			groupSample.printEditGroup(groupEditBean);
			System.out.println("\n");

			allGroups = groupSample.getAllGroups();
			System.out.println("After add:");
			groupSample.printGroups(allGroups);
			System.out.println("\n");

			PersonSample personSample = new PersonSample(groupSample.connector);
			String personID1 = personSample.createUser(1);
			String personID2 = personSample.createUser(2);
			groupSample.addPersonsToGroup(groupID, new String[] { personID1, personID2 });

			WSPersonBean[] personInGroups = groupSample.getPersonsForGroup(groupID);
			System.out.println("Person in group:" + groupID);
			personSample.printPersons(personInGroups);

			groupSample.removePersonsFromGroup(groupID, new String[] { personID1 });
			personInGroups = groupSample.getPersonsForGroup(groupID);
			System.out.println("Person in group:" + groupID + " afer remove first person");
			personSample.printPersons(personInGroups);

			groupSample.removePersonsFromGroup(groupID, new String[] { personID2 });
			personSample.deletePerson(personID1);
			personSample.deletePerson(personID2);

			groupSample.deleteGroup(groupID);
			System.out.println("After delete:");
			allGroups = groupSample.getAllGroups();
			groupSample.printGroups(allGroups);

		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
