package com.trackplus.track.ws.samples;

import com.trackplus.track.ws.beans.WSItemLinkBean;
import com.trackplus.track.ws.beans.WSKeyValueBean;
import com.trackplus.track.ws.beans.WSOptionBean;
import com.trackplus.track.ws.beans.WSParameter;
import com.trackplus.track.ws.tcl.TCLFacade;

/**
 */
public class ItemLinkSample extends ItemSample {

	/**
	 * The constructor.
	 */
	public ItemLinkSample() {
		super();
	}

	/**
	 * Constructor with existing connection.
	 * @param connector
	 */
	public ItemLinkSample(TCLFacade connector) {
		super(connector);
	}

	public WSItemLinkBean[] getItemLinks(String itemID){
		try{
			WSItemLinkBean[] links= connector.getItemLinks(itemID);
			return links;
		}catch (Exception ex){
			ex.printStackTrace();
			return null;
		}
	}

	private void printItemLinks(WSItemLinkBean[] links) {
		if(links!=null){
			for(int i=0;i<links.length;i++){
				System.out.println("Item link:"+i);
				WSItemLinkBean link=links[i];
				printItemLink(link);
				System.out.println("\n\n");
			}
		}
	}

	private void printItemLink(WSItemLinkBean itemLinkBean){
		System.out.println("objectID="+itemLinkBean.getObjectID());
		System.out.println("linkType="+itemLinkBean.getLinkType());
		System.out.println("linkTypeName="+itemLinkBean.getLinkTypeName());
		System.out.println("linkedWorkItemID="+itemLinkBean.getLinkedWorkItemID());
		System.out.println("linkedWorkItemTitle="+itemLinkBean.getLinkedWorkItemTitle());
//		System.out.println("editable="+itemLinkBean.getEditable());
		System.out.println("sortOrder="+itemLinkBean.getSortOrder());
		System.out.println("description="+itemLinkBean.getDescription());
		System.out.println("externalLink="+itemLinkBean.getExternalLink());
		System.out.println("changedBy="+itemLinkBean.getChangedBy());
		System.out.println("lastEdit="+itemLinkBean.getLastEdit());

		WSKeyValueBean[] parameters=itemLinkBean.getParameters();
		if(parameters==null||parameters.length==0){
			System.out.println("No params");
		}else{
			System.out.println("parameters="+parameters.length);
		}

	}

	public WSOptionBean[] getLinkTypes(){
		try{
			WSOptionBean[] linkTypes= connector.getLinkTypes();
			return linkTypes;
		}catch (Exception ex){
			ex.printStackTrace();
		}
		return null;
	}

	public void printOptions(WSOptionBean[] options){
		if(options!=null){
			for (int i = 0; i < options.length; i++) {
				System.out.println("ID=" + options[i].getObjectID() + " Label=" + options[i].getLabel());
			}
		}
	}

	public WSParameter[] getLinkTypeParams(String linkTypeID){
		try{
			WSParameter[] parameters=connector.getLinkTpeParameters(linkTypeID);
			return parameters;
		}catch (Exception ex){
			ex.printStackTrace();
		}
		return null;
	}
	public void printParameters(WSParameter[] parameters){
		if(parameters==null||parameters.length==0){
			System.out.println("NO params");
		}else{
			WSParameter parameter;
			System.out.println("params size:"+parameters.length);
			for(int i=0;i<parameters.length;i++){
				parameter=parameters[i];
				if (parameter != null) {
				System.out.println("parameter=" + parameter);
				System.out.println("name=" + parameter.getName());
				System.out.println("type=" + parameter.getType());
//				System.out.println("required=" + parameter.getRequired());

				WSOptionBean[] options= parameter.getOptions();
				System.out.println("options:");
				printOptions(options);
				System.out.println("\n\n");
			}
		}
	}
	}
	public String saveItemLink(String itemID, WSItemLinkBean itemLinkBean){
		try{
			return connector.saveItemLink(itemID,itemLinkBean);
		}catch (Exception ex){
			ex.printStackTrace();
			return  null;
		}
	}
	public WSItemLinkBean getItemLink(String itemID,String itemLinkID){
		try{
			return  connector.getItemLink(itemID,itemLinkID);
		}catch (Exception ex){
			ex.printStackTrace();
		}
		return null;
	}
	public void deleteItemLinks(String itemID,String[] itemLinkArray){
		try{
			connector.deleteItemLinks(itemID,itemLinkArray);
		}catch (Exception ex){
			ex.printStackTrace();
		}
	}


	public static void main(String[] args) {
		ItemLinkSample itemLinkSample=new ItemLinkSample();
		itemLinkSample.login(uName,pField);

		String itemID=itemLinkSample.createItem("My dummy item");
		// String itemID="3";

		if(itemID!=null){
			WSOptionBean[] linkTypes=itemLinkSample.getLinkTypes();
			System.out.println("LinkTypes:");
			itemLinkSample.printOptions(linkTypes);
			System.out.println("\n\n\n*********************\n\n\n\n");
			String linkTypeID=null;
			if(linkTypes!=null&&linkTypes.length>0){
				WSOptionBean linkType=linkTypes[0];
				linkTypeID=linkType.getObjectID();
				WSParameter[] parameters=itemLinkSample.getLinkTypeParams(linkTypeID);
				System.out.println("Parameters for link type:"+linkTypeID);
				itemLinkSample.printParameters(parameters);
				System.out.println("\n\n\n*********************\n\n\n\n");
			}

			//create item Link
			String newItemLinkID=null;
			if(linkTypeID!=null){
				WSItemLinkBean itemLinkBean=new WSItemLinkBean();
				String linkedItemID=itemLinkSample.createItem("LinkedItem");
				// String linkedItemID="65";
				if(linkedItemID!=null){
					itemLinkBean.setLinkType(linkTypeID);
					itemLinkBean.setLinkedWorkItemID(linkedItemID);
					itemLinkBean.setDescription("some item link form webservice");

					newItemLinkID=itemLinkSample.saveItemLink(itemID,itemLinkBean);
					System.out.print("ItemLink saved with id:"+newItemLinkID);
					itemLinkBean=itemLinkSample.getItemLink(itemID,newItemLinkID);
					System.out.println(" ITEM LINK bean:");
					itemLinkSample.printItemLink(itemLinkBean);
				}
			}

			//get item Links
			WSItemLinkBean[] links=itemLinkSample.getItemLinks(itemID);
			int size=links==null?0:links.length;
			System.out.println("All item links :size="+size);
			itemLinkSample.printItemLinks(links);


			//delete itemLinks
			String[] deletedItemLinks=new String[]{newItemLinkID};
			itemLinkSample.deleteItemLinks(itemID,deletedItemLinks);

			links=itemLinkSample.getItemLinks(itemID);
			size=links==null?0:links.length;
			System.out.println("AGAIN all item links after delete: size="+size);
			itemLinkSample.printItemLinks(links);

		}
	}

}
