/*******************************************************************************
 * Copyright (C) 2015 Steinbeis GmbH & Co. KG.
 * All rights reserved. 
 *
 * This program and the accompanying materials are made 
 * available under the terms of the Track+ License v2.0
 * which is available at http://www.trackplus.com/tpl-v20.html
 *
 * $Id:$
 *******************************************************************************/
package com.trackplus.track.ws.samples;

import java.rmi.RemoteException;
import java.util.Date;
import java.util.Iterator;
import java.util.Set;

import com.trackplus.track.ws.beans.WSAccountBean;
import com.trackplus.track.ws.beans.WSAttachmentBean;
import com.trackplus.track.ws.beans.WSBudgetBean;
import com.trackplus.track.ws.beans.WSComment;
import com.trackplus.track.ws.beans.WSCommentEdit;
import com.trackplus.track.ws.beans.WSConsInfEdit;
import com.trackplus.track.ws.beans.WSConsInfShow;
import com.trackplus.track.ws.beans.WSCostBean;
import com.trackplus.track.ws.beans.WSCustomFieldDisplayValue;
import com.trackplus.track.ws.beans.WSCustomFieldValue;
import com.trackplus.track.ws.beans.WSDisplayItemBean;
import com.trackplus.track.ws.beans.WSEditItemContextBean;
import com.trackplus.track.ws.beans.WSEstimatedBudgetBean;
import com.trackplus.track.ws.beans.WSField;
import com.trackplus.track.ws.beans.WSFieldsConfigBean;
import com.trackplus.track.ws.beans.WSHistoryValues;
import com.trackplus.track.ws.beans.WSItemBean;
import com.trackplus.track.ws.beans.WSItemContextBean;
import com.trackplus.track.ws.beans.WSItemHistoryBean;
import com.trackplus.track.ws.beans.WSLabelValueBean;
import com.trackplus.track.ws.beans.WSOptionBean;
import com.trackplus.track.ws.beans.WSPersonBean;
import com.trackplus.track.ws.beans.WSSortedOptionBean;
import com.trackplus.track.ws.beans.WSStateOptionBean;
import com.trackplus.track.ws.beans.WSTimeAndCost;
import com.trackplus.track.ws.beans.WSTreeNode;
import com.trackplus.track.ws.tcl.TCLFacade;
import com.trackplus.track.ws.tcl.TCLFacadeException;
import com.trackplus.track.ws.util.ItemWrapper;
import com.trackplus.track.ws.util.TCLUtil;


/**
 * <p>
 * This class serves as a little example how to work with the Track+ client
 * library (TCL). It demonstrates how to create an item, retrieve an item by
 * its identifier, and change various aspects of an item. It also
 * demonstrates how to access all item information.</p>
 * <p>
 * The class uses the {@link LoginSample} class to perform
 * the login. You need to set the URL and the user name and password in that
 * class before compiling and running this class here.</p>
 * <p>
 * This example is a Java console application. You can run it directly by
 * calling its <code>main()</code> method.</p>
 * 
 * 
 */
public class ItemSample extends LoginSample implements Runnable {
	
	private int itemCounter = 1;
	/**
	 * The constructor.
	 */
	public ItemSample() {
		super();
	}
	
	/**
	 * Constructor with existing connection.
	 * @param connector
	 */
	public ItemSample(TCLFacade connector) {
		super(connector);
	}

	/**
	 * Create an item, setting all required parameters.
	 * @return the new item id
	 */
	public String createItem(){
		++itemCounter;
		return createItem("WebService item " + itemCounter, 3);
	}
	
	/**
	 * Create an item, setting all required parameters.
	 * @return the new item id
	 */
	public String createItem(String title){
		return createItem(title, 3);
	}
	
	public String createItem(String title, int parentId){
		System.out.println("\n->ItemSample.createItem()");
		//to create an item we need an project and issueType
		String projectID=null;
		String issueTypeID=null;
		//getting the project for the user
		long start = startTime();
		try {
			WSLabelValueBean[] projects= connector.getProjects();
			if(projects==null||projects.length==0){
				System.out.println("No projects found for user: " + uName);
				return null;
			}
			stopTime("Got " + projects.length + " projects.");
			projectID=projects[0].getValue();
			
			WSLabelValueBean[] issueTypes=connector.getIssueTypes(projectID);
			if(issueTypes==null||issueTypes.length==0){
				System.out.println("No item types found for user: " + uName);
				return null;
			}
			stopTime("Got " + issueTypes.length + " item types");
			issueTypeID=issueTypes[0].getValue();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Error obtaining the project and item type.");
			handleTCLFacadeException(e);
			return null;
		}
		stopTime("Got projects and item types");
		
		WSEditItemContextBean editItemCtx=null;
		try {
			editItemCtx=connector.getNewItemContext(projectID, issueTypeID);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Error encountered obtaining the newItemContext");
			handleTCLFacadeException(e);
		}
		if (editItemCtx == null){
			return null;
		}
		WSItemBean item=editItemCtx.getItem();
		//the item contains the default values
		//print item, we need the custom fields
		printEditItemContext(editItemCtx,getCustomFields());
		
		//the fieldsConfig contains settings(label, tooltip, readOnly) and options only for fields 
		//present on the form configured for NEW item in given project and issueType
		//and if we change a field that is not present on the form will do not have effect.
		WSFieldsConfigBean cfg=editItemCtx.getFieldsConfig();
		
		//set some values
		item.setSynopsis(title);
		item.setDescription("This item " + itemCounter + " was created via the web service interface.");
		//set the priority( assume the priority is present on the form)
		//the priorities length must be >=1 because include the default value
		WSSortedOptionBean[] priorities=cfg.getPriorityOptions();
		String priorityID=priorities[priorities.length-1].getObjectID();
		item.setPriorityID(priorityID);
		//set the comment
		item.setNewComment("Comment from web service " + itemCounter + ".");
		
		WSCustomFieldValue[] customFieldValues=item.getCustomFieldValues();
		if(customFieldValues==null){
			System.out.println("Returned null for custom field values");
		}else{
			System.out.println("customFieldValues size="+customFieldValues.length);
		}
		
		item.setSuperiorWorkitem(String.valueOf(parentId));
		
		//save the item
		String itemID=null;
	    start = new Date().getTime();
		try {
			itemID = connector.saveItem(item);
			System.out.println("Item saved successfully: The new itemID is:"+itemID);
		} catch (RemoteException e) {
			System.out.println("Failed to save the item");
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Failed to save the item");
			handleTCLFacadeException(e);
		}
		long end = new Date().getTime();
		System.err.println("<-ItemSample.createItem() (" + (end-start) + " ms)\n");
		return itemID;
	}
	
	public void createItems(int numberOfItems, String projectID){
		if (numberOfItems < 100) {
			numberOfItems = 100;
		}
		//to create an item we need an project and issueType
		// String projectID=null;
		String issueTypeID=null;
		//getting the project for the user

		if (projectID == null) {
			try {
				WSLabelValueBean[] projects= connector.getProjects();
				if(projects==null||projects.length==0){
					System.out.println("No projects found for user: " + uName);
					return;
				}
				stopTime("Got " + projects.length + " projects.");
				projectID=projects[1].getValue();

			} catch (RemoteException e) {
				e.printStackTrace();
			} catch (TCLFacadeException e) {
				System.out.println("Error obtaining the project and item type.");
				handleTCLFacadeException(e);
				return;
			}
		}
		try {
			WSLabelValueBean[] issueTypes=connector.getIssueTypes(projectID);
			if(issueTypes==null||issueTypes.length==0){
				System.out.println("No item types found for user: " + uName);
				return;
			}
			issueTypeID=issueTypes[0].getValue();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Error obtaining the project and item type.");
			handleTCLFacadeException(e);
			return;
		}
		
		
		
		WSEditItemContextBean editItemCtx=null;
		try {
			editItemCtx=connector.getNewItemContext(projectID, issueTypeID);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Error encountered obtaining the newItemContext");
			handleTCLFacadeException(e);
		}
		if (editItemCtx == null){
			return;
		}
		WSItemBean item=editItemCtx.getItem();
		//the item contains the default values
		//print item, we need the custom fields
		// printEditItemContext(editItemCtx,getCustomFields());
		
		//the fieldsConfig contains settings(label, tooltip, readOnly) and options only for fields 
		//present on the form configured for NEW item in given project and issueType
		//and if we change a field that is not present on the form will do not have effect.
		WSFieldsConfigBean cfg=editItemCtx.getFieldsConfig();

		for (int i = 0; i < numberOfItems / 100 ; ++i) {
			System.out.println();
			System.out.print((i+1)*100);
			long start = new Date().getTime();
			for (int k = 0; k < 100; ++k) {

				//set some values
				item.setSynopsis("Web item no. " + i*100+k);
				item.setDescription("This item " + itemCounter + " was created via the web service interface.");
				//set the priority( assume the priority is present on the form)
				//the priorities length must be >=1 because include the default value
				WSSortedOptionBean[] priorities=cfg.getPriorityOptions();
				String priorityID=priorities[priorities.length-1].getObjectID();
				item.setPriorityID(priorityID);
				//set the comment
				item.setNewComment("Comment from web service " + itemCounter + ".");

				WSCustomFieldValue[] customFieldValues=item.getCustomFieldValues();

				// item.setSuperiorWorkitem(String.valueOf(3));

				//save the item
				String itemID=null;
				try {
					itemID = connector.saveItem(item);
				} catch (RemoteException e) {
					System.out.println("Failed to save the item");
					e.printStackTrace();
				} catch (TCLFacadeException e) {
					System.out.println("Failed to save the item");
					handleTCLFacadeException(e);
				}
			}
			
			long end = new Date().getTime();
			long ms = end - start;
			System.out.print(": " + ms/100 + "ms per item.");
		}

		return;
	}
	
	/**
	 * Retrieve an item by its identifier. This retrieves the item,
	 * but all references to other objects are just available as foreign
	 * keys. For example, the priority is not returned as "high" but rather
	 * as "1". The manager is not returned as "John Manager" but as "238".
	 * 
	 * @param itemID the object identifier of the item to retrieve.
	 * @return an item bean
	 */
	public WSItemBean getItem(String itemID){
		System.out.println("\n->ItemSample.getItem()");
		WSItemBean itemBean=null;
		try {
			itemBean = connector.getItem(itemID);
			printItem(itemBean);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Failed to obtain the item");
			handleTCLFacadeException(e);
		}
		System.out.println("<-ItemSample.getItem()\n");
		return itemBean;
	}
	
	/**
	 * Retrieve an item by its identifier. All references to other objects
	 * such as persons or priorities are resolved to clear text, so that
	 * the value can be displayed directly to some user interface.
	 * 
	 * @param itemID
	 * @return a display item bean
	 */
	public WSDisplayItemBean getDisplayItem(String itemID){
		System.out.println("\n->ItemSample.getDisplayItem()");
		WSDisplayItemBean displayItem=null;
		try {
			displayItem = connector.getDisplayItem(itemID);
			printDisplayItem(displayItem);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Failed to obtain the item");
			handleTCLFacadeException(e);
		}
		System.out.println("<-ItemSample.getDisplayItem()\n");
		return displayItem;
	}
	
	/**
	 * <p>The item context contains all objects the item proper references.
	 * For example, the context contains the persons that are the (possible) manager
	 * and responsible of that item, with all their related information,
	 * or the possible statuses, priorities, severities, and so on.</p>
	 * <p>
	 * The context is helpful when constructing the selection boxes at the
	 * user interface.
	 * </p> 
	 * @param itemID
	 * @return a bean with the item context
	 */
	public WSItemContextBean getItemContext(String itemID){
		System.out.println("\n->ItemSample.getItemContext()");
		WSItemContextBean ctx=null;
		try {
			ctx = connector.getItemContext(itemID);
			printItemContext(ctx);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Failed to obtain the item context");
			handleTCLFacadeException(e);
		}
		System.out.println("<-ItemSample.getItemContext()\n");
		return ctx;
	}
	public WSItemContextBean[] getItemContexts(String[] itemIDs){
		System.out.println("\n->ItemSample.getItemContexts()");
		WSItemContextBean ctxs[]=null;
		try {
			ctxs = connector.getItemContexts(itemIDs);
			if(ctxs!=null){
				for(int i=0;i<ctxs.length;i++){
					printItemContext(ctxs[i]);
				}
			}
			
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Failed to obtain the item context");
			handleTCLFacadeException(e);
		}
		System.out.println("<-ItemSample.getItemContext()\n");
		return ctxs;
	}


	/**
	 * <p>The item context contains all objects the item proper references.
	 * For example, the context contains the persons that are the (possible) manager
	 * and responsible of that item, with all their related information,
	 * or the possible statuses, priorities, severities, and so on.</p>
	 * <p>
	 * The context is helpful when constructing the selection boxes at the
	 * user interface.
	 * </p> 
	 * @param itemID
	 * @return a bean to edit an item
	 */
	public WSEditItemContextBean getEditItemContext(String itemID){
		System.out.println("\n->ItemSample.getEditItemContext()");
		WSEditItemContextBean editCtx=null;
		try {
			editCtx = connector.getEditItemContext(itemID);
			printEditItemContext(editCtx,getCustomFields());
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Failed to obtain the item context");
			handleTCLFacadeException(e);
		}
		System.out.println("<-ItemSample.getEditItemContext()\n");
		return editCtx;
	}
	
	/**
	 * This method demonstrates how to change some aspects of an item.
	 * 
	 * @param itemID
	 */
	public void modifyItem(String itemID){
		System.out.println("\n->ItemSample.modifyItem()");
		WSEditItemContextBean editCtx=null;
		try {
			editCtx = connector.getEditItemContext(itemID);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Failed to obtain the item context");
			handleTCLFacadeException(e);
		}
		if(editCtx==null){
			return;
		}
		WSItemBean item=editCtx.getItem();
		WSFieldsConfigBean cfg=editCtx.getFieldsConfig();
		
		//modify the description
		item.setDescription("Description modified via the web service interface.");

		//modify the startDate
		Date startDate=new Date();
		String startDateISO=TCLUtil.formatISODate(startDate);
		item.setStartDate(startDateISO);

		//modify the severity
		WSSortedOptionBean[] severites=cfg.getSeverityOptions();
		//take the last severity
		if(severites!=null){
			String severityID=severites[severites.length-1].getObjectID();
			item.setSeverityID(severityID);
		}
		
		//modify the responsible
		WSPersonBean[] resposibleOptions=cfg.getResponsibleOptions();
		String responsibleID=resposibleOptions[0].getPersonID();
		item.setResponsibleID(responsibleID);
		
		//save the item
		try {
			connector.saveItem(item);
			System.out.println("Item modified successfully");
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Fail to save the item");
			handleTCLFacadeException(e);
		}
		
		System.out.println("<-ItemSample.modifyItem()\n");
	}
	
	/**
	 * This method demonstrates how to add a comment to an item.
	 * @param itemID
	 */
	public void addComment(String itemID){
		System.out.println("\n->ItemSample.addComment()");
		WSCommentEdit newComment=new WSCommentEdit();
		newComment.setWorkItemID(itemID);
		newComment.setDescription("Some new comment via the web-service");
		try {
			connector.editComment(newComment);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		
		WSCommentEdit[] comments=null;
		try {
			comments=connector.getComments(itemID);
		} catch (RemoteException e1) {
			e1.printStackTrace();
		} catch (TCLFacadeException e1) {
			handleTCLFacadeException(e1);
		}
		printComments(comments);
		System.out.println("<-ItemSample.addComment()\n");
	}
	public void editComment(String itemID){
		System.out.println("->ItemSample.editComment()");
		WSCommentEdit[] comments=null;
		try {
			comments=connector.getComments(itemID);
		} catch (RemoteException e1) {
			e1.printStackTrace();
		} catch (TCLFacadeException e1) {
			handleTCLFacadeException(e1);
		}
		if(comments==null||comments.length==0){
			System.out.println("No comments found");
			return;
		}
		System.out.println("original comments");
		printComments(comments);
		WSCommentEdit comment=comments[0];
		comment.setDescription("Some comment modification");
		try {
			connector.editComment(comment);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		
		try {
			comments=connector.getComments(itemID);
		} catch (RemoteException e1) {
			e1.printStackTrace();
		} catch (TCLFacadeException e1) {
			handleTCLFacadeException(e1);
		}
		System.out.println("Modified comments");
		printComments(comments);
		System.out.println("<-ItemSample.editComment()");
	}
	public void deleteComments(String itemID){
		System.out.println("->ItemSample.deleteComments()");
		WSCommentEdit[] comments=null;
		try {
			comments=connector.getComments(itemID);
		} catch (RemoteException e1) {
			e1.printStackTrace();
		} catch (TCLFacadeException e1) {
			e1.printStackTrace();
		}
		if(comments==null||comments.length==0){
			System.out.println("No comments found");
			return;
		}
		System.out.println("Original comments");
		printComments(comments);
		WSCommentEdit comment=comments[0];
		String[] commentIDs=new String[]{comment.getObjectID()};
		try {
			connector.deleteComments(itemID, commentIDs);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		
		try {
			comments=connector.getComments(itemID);
		} catch (RemoteException e1) {
			e1.printStackTrace();
		} catch (TCLFacadeException e1) {
			e1.printStackTrace();
		}
		System.out.println("Modified comments");
		printComments(comments);
		System.out.println("<-ItemSample.deleteComments()");
	}
	/**
	 * This method demonstrates how to change the item status.
	 * 
	 * @param itemID
	 */
	public void changeStatus(String itemID){
		System.out.println("\n->ItemSample.changeStatus()");
		WSEditItemContextBean editCtx=null;
		try {
			editCtx = connector.getEditItemContext(itemID);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Failed to obtain the item context");
			handleTCLFacadeException(e);
		}
		if(editCtx==null){
			return;
		}
		WSItemBean item=editCtx.getItem();
		WSFieldsConfigBean cfg=editCtx.getFieldsConfig();
		
		//set the new status
		WSStateOptionBean[] states=cfg.getStateOptions();
		String stateID=states[states.length-1].getObjectID();
		item.setStateID(stateID);
		
		//set comment
		item.setNewComment("Status changed via web-service");
		
		//save the item
		try {
			connector.saveItem(item);
			System.out.println("Item modified successfully");
			WSEditItemContextBean editItemCtx=getEditItemContext(itemID);
			printEditItemContext(editItemCtx, getCustomFields());
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Failed to save the item");
			handleTCLFacadeException(e);
		}
		System.out.println("->ItemSample.changeStatus()\n");
	}
	
	/**
	 * This method obtains the complete item history.
	 * 
	 * @param itemID
	 */
	public void getHistory(String itemID){
		System.out.println("\n->ItemSample.getHistory()");
		WSItemHistoryBean itemHistory=null;
		try {
			itemHistory = connector.getItemHistory(itemID);
			printItemHistory(itemHistory);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Failed to obtain the item history");
			handleTCLFacadeException(e);
		}
		System.out.println("<-ItemSample.getHistory()\n");
	}
	
	/**
	 * This method demonstrates how to mark an item as deleted. To really
	 * remove it from the database you have to delete the workspace containing it.
	 * 
	 * @param itemID
	 */
	public void deleteItem(String itemID){
		System.out.println("\n->ItemSample.deleteItem()");
		try {
			connector.deletItem(itemID);
			System.out.println("Item after delete:");
			WSItemBean itemBean = connector.getItem(itemID);
			printItemSimple(itemBean);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Failed to obtain the item");
			handleTCLFacadeException(e);
		}
		System.out.println("<-ItemSample.deleteItem()\n");
	}
	
	/**
	 * This method shows how to restore an item that has been marked
	 * as deleted previously.
	 * 
	 * @param itemID
	 */
	public void undeleteItem(String itemID){
		System.out.println("\n->ItemSample.undeleteItem()");
		try {
			connector.undeleteItem(itemID);
			System.out.println("Item after undeleteItem:");
			WSItemBean itemBean = connector.getItem(itemID);
			printItemSimple(itemBean);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Failed to obtain the item");
			handleTCLFacadeException(e);
		}
		System.out.println("<-ItemSample.undeleteItem()\n");
	}
	
	/**
	 * This method shows how to archive an item. It will then disappear
	 * from most places where you can get items.
	 * 
	 * @param itemID
	 */
	public void archiveItem(String itemID){
		System.out.println("\n->ItemSample.archiveItem()");
		try {
			connector.archiveItem(itemID);
			System.out.println("Item after archive:");
			WSItemBean itemBean = connector.getItem(itemID);
			printItemSimple(itemBean);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Failed to obtain the item");
			handleTCLFacadeException(e);
		}
		System.out.println("<-ItemSample.archiveItem()\n");
	}
	
	/**
	 * This method shows how to restore an archived item.
	 * 
	 * @param itemID
	 */
	public void unarchiveItem(String itemID){
		System.out.println("\n->ItemSample.unarchiveItem()");
		try {
			connector.unarchiveItem(itemID);
			System.out.println("Item after unarchive:");
			WSItemBean itemBean = connector.getItem(itemID);
			printItemSimple(itemBean);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			System.out.println("Failed to obtain the item");
			handleTCLFacadeException(e);
		}
		System.out.println("<-ItemSample.unarchiveItem()\n");
	}
	
	
	/**
	 * This method demonstrates how to obtain the various parts of the
	 * history and prepare them for output to a user interface.
	 * 
	 * @param itemHistory
	 */
	private void printItemHistory(WSItemHistoryBean itemHistory) {
		if(itemHistory==null){
			System.out.println("No history");
			return;
		}
		//history
		WSHistoryValues[] historyValues=itemHistory.getHistoryValues();
		System.out.println("History");
		if(historyValues==null||historyValues.length==0){
			System.out.println("\tNo history");
		}else{
			for (int i = 0; i < historyValues.length; i++) {
				System.out.println("\t"+"By "+historyValues[i].getChangedByName()+" At "+historyValues[i].getLastEdit());
				System.out.println("\t"+historyValues[i].getFieldName());
				System.out.println("\t newValue="+historyValues[i].getNewValue());
				System.out.println("\t oldValue="+historyValues[i].getOldValue()+"\n");
			}
		}
		//The cost changes history
		WSCostBean[] costs=itemHistory.getCosts();
		System.out.println("Cost history");
		if(costs==null||costs.length==0){
			System.out.println("\tNo cost change");
		}else{
			for (int i = 0; i < costs.length; i++) {
				System.out.println("\t"+"By "+costs[i].getCreatedByName()+" At "+costs[i].getLastEdit()+" Account="+costs[i].getAccountName());
				System.out.println("\t"+"Cost:"+costs[i].getCost()+" Hours:"+costs[i].getHours());
				System.out.println("\t"+costs[i].getDescription()+"\n");
			}
		}
	}
	
	
	public WSField[] getCustomFields(){
		try {
			return connector.getCustomFields();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		return new WSField[0];
	}
	
	public void printItemSimple(WSItemBean item) {
		System.out.println("-------WSItemBean-----------------------");
		printBaseFields(item);
		System.out.println("----------------------------------------\n");
	}
	
	public void printItem(WSItemBean item) {
		System.out.println("\n-------WSItemBean-----------------------");
		
		printBaseFields(item);
		System.out.println();
		
		printCustomFields(item.getCustomFieldValues());
		printChildrenIds(item.getChildrenIds());
		printAttachments(item.getAttachments());
		printComments(item.getComments());
		printConsInf(item.getConsInfEdit());
		printTimeAndCost(item.getTimeAndCost());
		
		System.out.println("----------------------------------------\n");
		
	}
	public void printBaseFields(WSItemBean item) {
		System.out.println("ChangedByID="+item.getChangedByID());
		System.out.println("Created="+item.getCreated());
		System.out.println("Description="+item.getDescription());
		System.out.println("EndDate="+item.getEndDate());
		System.out.println("IssueTypeID="+item.getIssueTypeID());
		System.out.println("LastEdit="+item.getLastEdit());
		System.out.println("ManagerID="+item.getManagerID());
		System.out.println("NewComment="+item.getNewComment());
		System.out.println("ObjectID="+item.getObjectID());
		System.out.println("OriginatorID="+item.getOriginatorID());
		System.out.println("PriorityID="+item.getPriorityID());
		System.out.println("PrivateItem="+item.getPrivateItem());
		System.out.println("ProjectID="+item.getProjectID());
		System.out.println("ReleaseNoticedID="+item.getReleaseNoticedID());
		System.out.println("ReleaseScheduledID="+item.getReleaseScheduledID());
		System.out.println("ResponsibleID="+item.getResponsibleID());
		System.out.println("SeverityID="+item.getSeverityID());
		System.out.println("StartDate="+item.getStartDate());
		System.out.println("StateID="+item.getStateID());
		System.out.println("SuperiorWorkitem="+item.getSuperiorWorkitem());
		System.out.println("SuperiorWorkitemSynopsys="+item.getSuperiorWorkitemSynopsys());
		System.out.println("Synopsis="+item.getSynopsis());
		// System.out.println("Archived="+item.getArchived());
		// System.out.println("Deleted="+item.getDeleted());
	}
	
	
	public void printDisplayItem(WSDisplayItemBean displayItem) {
		System.out.println("\n-------WSDisplayItemBean----------------");
		WSPersonBean changedBy = displayItem.getChangedBy();
		System.out.println(displayItem.getChangedByLabel()+"="+(changedBy==null?null:changedBy.getPersonID()+":"+changedBy.getFullName()));
		

		System.out.println(displayItem.getCreatedLabel()+"="+displayItem.getCreated());
		System.out.println(displayItem.getDescriptionLabel()+"="+displayItem.getDescription());
		System.out.println(displayItem.getEndDateLabel()+"="+displayItem.getEndDate());
		
		WSSortedOptionBean issueType = displayItem.getIssueType();
		System.out.println(displayItem.getIssueTypeLabel()+"="+(issueType==null?null:issueType.getLabel()));
		
		System.out.println(displayItem.getLastEditLabel()+"="+displayItem.getLastEdit());
		
		WSPersonBean manager = displayItem.getManager();
		System.out.println(displayItem.getManagerLabel()+"="+(manager==null?null:manager.getPersonID()+":"+manager.getFullName()));
		
		//System.out.println(displayItem.getNewCommentLabel()+"="+displayItem.getNewComment());
		System.out.println(displayItem.getObjectIDLabel()+"="+displayItem.getObjectID());
		
		WSPersonBean originator = displayItem.getOriginator();
		System.out.println(displayItem.getOriginatorLabel()+"="+(originator==null?null:originator.getPersonID()+":"+originator.getFullName()));
		
		WSSortedOptionBean priority = displayItem.getPriority();
		System.out.println(displayItem.getPriorityLabel()+"="+(priority==null?null:priority.getObjectID()+":"+priority.getLabel()));
		
		System.out.println("private="+displayItem.getPrivateItem());
		
		WSOptionBean project = displayItem.getProject();
		System.out.println(displayItem.getProjectLabel()+"="+(project==null?null:project.getObjectID()+":"+project.getLabel()));
		
		WSSortedOptionBean releaseNoticed = displayItem.getReleaseNoticed();
		System.out.println(displayItem.getReleaseNoticedLabel()+"="+(releaseNoticed==null?null:releaseNoticed.getObjectID()+":"+releaseNoticed.getLabel()));
		
		WSSortedOptionBean releaseScheduled = displayItem.getReleaseScheduled();
		System.out.println(displayItem.getReleaseScheduledLabel()+"="+(releaseScheduled==null?null:releaseScheduled.getObjectID()+":"+releaseScheduled.getLabel()));
		
		WSPersonBean responsible = displayItem.getResponsible();
		System.out.println(displayItem.getResponsibleLabel()+"="+(responsible==null?null:responsible.getPersonID()+":"+responsible.getFullName()));
		
		WSSortedOptionBean severity = displayItem.getSeverity();
		System.out.println(displayItem.getSeverityLabel()+"="+(severity==null?null:severity.getObjectID()+":"+severity.getLabel()));
		
		System.out.println(displayItem.getStartDateLabel()+"="+displayItem.getStartDate());
		
		WSStateOptionBean state = displayItem.getState();
		System.out.println(displayItem.getStateLabel()+"="+(state==null?null:state.getObjectID()+":"+state.getLabel()));
		
		System.out.println(displayItem.getSuperiorWorkitemLabel()+"="+displayItem.getSuperiorWorkitem());
		System.out.println("Synopsis="+displayItem.getSuperiorWorkitemSynopsys());
		System.out.println(displayItem.getSynopsisLabel()+"="+displayItem.getSynopsis());
		System.out.println();
		
		printCustomDisplayFields(displayItem.getCustomFieldValues());
		printCustomFieldsLabels(displayItem.getCustomFieldLabels());
		printChildrenIds(displayItem.getChildrenIds());
		printAttachments(displayItem.getAttachments());
		printComments(displayItem.getComments());
		printConsInf(displayItem.getConsInfShow());
		printTimeAndCost(displayItem.getTimeAndCost());
		
		System.out.println("----------------------------------------\n");
	}
	
	public void printEditItemContext(WSEditItemContextBean editCtx,WSField[] customFields) {
		System.out.println("\n----------------------------------------");
		//ATTENTION the setting and options are present ONLY for fields present on form 
		//WSFieldsConfigBean cfg=editCtx.getFieldsConfig();
		WSItemBean item=editCtx.getItem();
		//create an wrapper over editItemContext
		ItemWrapper itemWrapper=new ItemWrapper(customFields,editCtx);
		
		System.out.println("---");
		printOptionsContext(editCtx.getFieldsConfig());
		//FormSample.printForm(editCtx.getForm());
		Set<String> presentFields=itemWrapper.getPresentFields();
		Iterator<String> it=presentFields.iterator();
		while(it.hasNext()){
			String fieldID=it.next();
			System.out.print("FieldID="+fieldID);
			//System.out.print("\tType="+itemWrapper.get);
			System.out.print(" Label="+itemWrapper.getLabel(fieldID));
			System.out.print(" Value="+itemWrapper.getValue(fieldID));
			System.out.print(" Display Value="+itemWrapper.getDisplayValue(fieldID));
			System.out.print(" ReadOnly="+itemWrapper.isReadOnly(fieldID));
			System.out.println();
		}
		
		printCustomFields(item.getCustomFieldValues());
		printChildrenIds(item.getChildrenIds());
		printAttachments(item.getAttachments());
		printComments(item.getComments());
		printConsInf(item.getConsInfEdit());
		printTimeAndCost(item.getTimeAndCost());
		
		
		
		System.out.println("----------------------------------------\n");
	}
	private void printOptionsContext(WSFieldsConfigBean cfg) {
		System.out.println("----------------------------------------\n");
		WSSortedOptionBean[] issueTypeOptions=cfg.getIssueTypeOptions();
		WSSortedOptionBean[] priorityOptions=cfg.getPriorityOptions();
		WSTreeNode[] releaseNoticedOptions=cfg.getReleaseNoticedOptions();
		WSTreeNode[] releaseScheduledOptions=cfg.getReleaseScheduledOptions();
		if(releaseNoticedOptions!=null){
			System.out.println("releaseNoticedOptions:");
			for (int i = 0; i < releaseNoticedOptions.length; i++) {
				BaseOptionSample.printNode(releaseNoticedOptions[i],"");
			}
			System.out.println("---------");
		}
		if(releaseScheduledOptions!=null){
			System.out.println("releaseScheduledOptions:");
			for (int i = 0; i < releaseScheduledOptions.length; i++) {
				BaseOptionSample.printNode(releaseScheduledOptions[i],"");
			}
			System.out.println("---------");
		}
		
		System.out.println("----------------------------------------\n");
	}
	
	private void printItemContext(WSItemContextBean ctx) {
		WSItemBean item=ctx.getItem();
		WSFieldsConfigBean cfg=ctx.getFieldsConfig();
		System.out.println("\n\n----------------------------------------");
		System.out.println(cfg.getChangedBySettings().getLabel()+"="+TCLUtil.findDisplayValue(cfg.getChangedByOptions(),item.getChangedByID()));
		//System.out.println(cfg.getClassSettings().getLabel()+"="+TCLUtil.findDisplayValue(cfg.getClassOptions(), item.getClassID()));
		System.out.println(cfg.getCreatedSettings().getLabel()+"="+item.getCreated());
		System.out.println(cfg.getDescriptionSettings().getLabel()+"="+item.getDescription());
		System.out.println(cfg.getEndDateSettings().getLabel()+"="+item.getEndDate());
		System.out.println(cfg.getIssueTypeSettings().getLabel()+"="+TCLUtil.findDisplayValue(cfg.getIssueTypeOptions(), item.getIssueTypeID()));
		System.out.println(cfg.getLastEditSettings().getLabel()+"="+item.getLastEdit());
		System.out.println(cfg.getManagerSettings().getLabel()+"="+TCLUtil.findDisplayValue(cfg.getManagerOptions(),item.getManagerID()));
		System.out.println(cfg.getCommentSettings().getLabel()+"="+item.getNewComment());
		//System.out.println(cfg.getIssueNoSettings().getLabel()+"="+item.getObjectID());
		System.out.println(cfg.getOriginatorSettings().getLabel()+"="+TCLUtil.findDisplayValue(cfg.getOriginatorOptions(),item.getOriginatorID()));
		System.out.println(cfg.getPrioritySettings().getLabel()+"="+TCLUtil.findDisplayValue(cfg.getPriorityOptions(),item.getPriorityID()));
		//no localization label for private from WSFieldsConfigBean
		System.out.println("Private="+item.getPrivateItem());
		System.out.println(cfg.getProjectSettings().getLabel()+"="+TCLUtil.findDisplayValue(cfg.getProjectOptions(), item.getProjectID()));
		System.out.println(cfg.getReleaseNoticedSettings().getLabel()+"="+TCLUtil.findDisplayValue(cfg.getReleaseNoticedOptions(), item.getReleaseNoticedID()));
		System.out.println(cfg.getReleaseScheduledSettings().getLabel()+"="+TCLUtil.findDisplayValue(cfg.getReleaseScheduledOptions(),item.getReleaseScheduledID()));
		System.out.println(cfg.getResponsibleSettings().getLabel()+"="+TCLUtil.findDisplayValue(cfg.getResponsibleOptions(), item.getResponsibleID()));
		//System.out.println(cfg.getSeveritySettings().getLabel()+"="+TCLUtil.findDisplayValue(cfg.getSeverityOptions(),item.getSeverityID()));
		System.out.println(cfg.getStartDateSettings().getLabel()+"="+item.getStartDate());
		System.out.println(cfg.getStateSettings().getLabel()+"="+TCLUtil.findDisplayValue(cfg.getStateOptions(), item.getStateID()));
		System.out.println(cfg.getSuperiorWorkitemSettings().getLabel()+"="+item.getSuperiorWorkitem()+" "+item.getSuperiorWorkitemSynopsys());
		//System.out.println(cfg.getSynopsysSettings().getLabel()+"="+item.getSynopsis());
		System.out.println();
		
		printCustomFields(item.getCustomFieldValues());
		printChildrenIds(item.getChildrenIds());
		printAttachments(item.getAttachments());
		printComments(item.getComments());
		printConsInf(item.getConsInfEdit());
		printTimeAndCost(item.getTimeAndCost());
		
		System.out.println("----------------------------------------\n\n");
	}
	
	private void printTimeAndCost(WSTimeAndCost timeAndCost) {
		if(timeAndCost==null){
			System.out.println("No time and cost");
		}else{
			System.out.println("Time and cost:");
			String currency=timeAndCost.getCurrency();
			System.out.println("\tCurrency:"+currency);
			String hoursperWorkDay=timeAndCost.getHoursPerWorkDay();
			System.out.println("\tHours per work day="+hoursperWorkDay);
			WSAccountBean[] accounts=timeAndCost.getAcounts();
			if(accounts==null||accounts.length==0){
				System.out.println("\tNo accounts:");
			}else{
				System.out.println("\tAccounts:");
				for (int i = 0; i < accounts.length; i++) {
					System.out.println("\t\t"+accounts[i].getObjectID()+" "+accounts[i].getFullName());
				}
			}
			WSBudgetBean budgetBean=timeAndCost.getBudget();
			if(budgetBean==null){
				System.out.println("\tNo budget");
			}else{
				System.out.println("\tBudget");
				System.out.println("\t\tChangedByID="+budgetBean.getChangedByID());
				System.out.println("\t\tChangedByName="+budgetBean.getChangedByName());
				System.out.println("\t\tDescription="+budgetBean.getDescription());
				System.out.println("\t\tLastEdit="+budgetBean.getLastEdit());
				System.out.println("\t\tObjectID="+budgetBean.getObjectID());
				System.out.println("\t\tTimeUnit="+budgetBean.getTimeUnit());
				System.out.println("\t\tTotalCost="+budgetBean.getTotalCost());
				System.out.println("\t\tTotalHours="+budgetBean.getTotalHours());
				//System.out.println("\t\tWorkItemID="+budgetBean.getWorkItemID());
			}
			WSEstimatedBudgetBean estimatedBudgetBean=timeAndCost.getEstimatedBudget();
			if(estimatedBudgetBean==null){
				System.out.println("\tNo Estimated Budget");
			}else{
				System.out.println("\tEstimated Budget");
				System.out.println("\t\tChangedByID="+estimatedBudgetBean.getChangedByID());
				System.out.println("\t\tChangedByName="+estimatedBudgetBean.getChangedByName());
				System.out.println("\t\tLastEdit="+estimatedBudgetBean.getLastEdit());
				System.out.println("\t\tObjectID="+estimatedBudgetBean.getObjectID());
				System.out.println("\t\tTimeUnit="+estimatedBudgetBean.getTimeUnit());
				System.out.println("\t\tEstimatedCost="+estimatedBudgetBean.getEstimatedCost());
				System.out.println("\t\tEstimatedHours="+estimatedBudgetBean.getEstimatedHours());
				//System.out.println("\t\tWorkItemID="+budgetBean.getWorkItemID());
			}
			WSCostBean[] costs=timeAndCost.getCosts();
			if(costs==null||costs.length==0){
				System.out.println("\tNo costs");
			}else{
				System.out.println("\tCosts:");
				for (int i = 0; i < costs.length; i++) {
					System.out.println("\t\t"+costs[i].getObjectID()+
							" Account="+costs[i].getAccountName()+
							" Cost="+costs[i].getCost()+
							" Time="+costs[i].getHours()+
							" Effortdate="+costs[i].getEffortdate()+
							" By "+costs[i].getCreatedByName()+
							" At "+costs[i].getLastEdit());
				}
			}
		}
	}
	
	
	private void printChildrenIds(String[] childrenIds) {
		if(childrenIds==null||childrenIds.length==0){
			System.out.println("No children");
		}else{
			System.out.print("Children:");
			for (int i = 0; i < childrenIds.length; i++) {
				System.out.print(childrenIds[i]+";");
			}
			System.out.println("\n");
		}
	}
	
	
	private void printCustomFields(WSCustomFieldValue[] customFields) {
		if(customFields==null||customFields.length==0){
			System.out.println("No custom fields");
		}else{
			System.out.println("Custom fields:");
			for (int i = 0; i < customFields.length; i++) {
				System.out.println("\t"+customFields[i].getFieldId()+"="+customFields[i].getValue());
			}
		}
	}
	
	
	private void printCustomDisplayFields(WSCustomFieldDisplayValue[] customFieldValues) {
		if(customFieldValues==null||customFieldValues.length==0){
			System.out.println("No custom fields");//
		}else{
			System.out.println("Custom fields:");
			for (int i = 0; i < customFieldValues.length; i++) {
				System.out.println("\t"+customFieldValues[i].getFieldId()+ " value="+customFieldValues[i].getValue()+" displayValue="+customFieldValues[i].getDisplayValue());
			}
		}
	}
	
	
	private void printCustomFieldsLabels(WSLabelValueBean[] customFieldLabels) {
		if(customFieldLabels==null||customFieldLabels.length==0){
			System.out.println("No custom fields");//
		}else{
			System.out.println("Custom fields labels:");
			for (int i = 0; i < customFieldLabels.length; i++) {
				System.out.println("\tFieldID="+customFieldLabels[i].getValue()+" Label="+customFieldLabels[i].getLabel());
			}
		}
		
	}
	
	
	private void printConsInf(WSConsInfEdit consInfEdit) {
		WSPersonBean[] consGroups=consInfEdit.getConsultantGroups();
		WSPersonBean[] consPersons=consInfEdit.getConsultantPersons();
		WSLabelValueBean[] selectedConsGroups=consInfEdit.getSelectedConsultantGroups();
		WSLabelValueBean[] selectedConsPersons=consInfEdit.getSelectedConsultantPersons();
		System.out.println("Consultant");
		if(consGroups==null||consGroups.length==0){
			System.out.println("\tNo posible groups:");
		}else{
			System.out.println("\tPosible groups:");
			for (int i = 0; i < consGroups.length; i++) {
				System.out.println("\t\t"+consGroups[i].getPersonID()+":"+consGroups[i].getFullName());
			}
		}
		if(consPersons==null||consPersons.length==0){
			System.out.println("\tNo posible persons:");
		}else{
			System.out.println("\tPosible persons:");
			for (int i = 0; i < consPersons.length; i++) {
				System.out.println("\t\t"+consPersons[i].getPersonID()+":"+consPersons[i].getFullName());
			}
		}
		if(selectedConsGroups==null||selectedConsGroups.length==0){
			System.out.println("\tNo Selected groups:");
		}else{
			System.out.println("\tSelected groups:");
			for (int i = 0; i < selectedConsGroups.length; i++) {
				System.out.println("\t\t"+selectedConsGroups[i].getValue()+":"+selectedConsGroups[i].getLabel());
			}
		}
		if(selectedConsPersons==null||selectedConsPersons.length==0){
			System.out.println("\tNo Selected persons:");
		}else{
			System.out.println("\tSelected persons:");
			for (int i = 0; i < selectedConsPersons.length; i++) {
				System.out.println("\t\t"+selectedConsPersons[i].getValue()+":"+selectedConsPersons[i].getLabel());
			}
		}
		WSPersonBean[] infGroups=consInfEdit.getInformantGroups();
		WSPersonBean[] infPersons=consInfEdit.getInformantPersons();
		WSLabelValueBean[] selectedInfGroups=consInfEdit.getSelectedInformantGroups();
		WSLabelValueBean[] selectedInfPersons=consInfEdit.getSelectedInformantPersons();
		System.out.println("Informants");
		if(infGroups==null||infGroups.length==0){
			System.out.println("\tNo posible groups:");
		}else{
			System.out.println("\tPosible groups:");
			for (int i = 0; i < infGroups.length; i++) {
				System.out.println("\t\t"+infGroups[i].getPersonID()+":"+infGroups[i].getFullName());
			}
		}
		if(infPersons==null||infPersons.length==0){
			System.out.println("\tNo posible persons:");
		}else{
			System.out.println("\tPosible persons:");
			for (int i = 0; i < infPersons.length; i++) {
				System.out.println("\t\t"+infPersons[i].getPersonID()+":"+infPersons[i].getFullName());
			}
		}
		if(selectedInfGroups==null||selectedInfGroups.length==0){
			System.out.println("\tNo Selected groups:");
		}else{
			System.out.println("\tSelected groups:");
			for (int i = 0; i < selectedInfGroups.length; i++) {
				System.out.println("\t\t"+selectedInfGroups[i].getValue()+":"+selectedInfGroups[i].getLabel());
			}
		}
		if(selectedInfPersons==null||selectedInfPersons.length==0){
			System.out.println("\tNo Selected persons:");
		}else{
			System.out.println("\tSelected persons:");
			for (int i = 0; i < selectedInfPersons.length; i++) {
				System.out.println("\t\t"+selectedInfPersons[i].getValue()+":"+selectedInfPersons[i].getLabel());
			}
		}
	}
	
	
	private void printConsInf(WSConsInfShow consInfShow) {
		WSPersonBean[] consGroups=consInfShow.getConsultantGroups();
		WSPersonBean[] consPersons=consInfShow.getConsultantPersons();
		System.out.println("Consultant");
		if(consGroups==null||consGroups.length==0){
			System.out.println("\tNo groups:");
		}else{
			System.out.println("\tGroups:");
			for (int i = 0; i < consGroups.length; i++) {
				System.out.println("\t\t"+consGroups[i].getPersonID()+":"+consGroups[i].getFullName());
			}
		}
		if(consPersons==null||consPersons.length==0){
			System.out.println("\tNo persons:");
		}else{
			System.out.println("\tPersons:");
			for (int i = 0; i < consPersons.length; i++) {
				System.out.println("\t\t"+consPersons[i].getPersonID()+":"+consPersons[i].getFullName());
			}
		}
		WSPersonBean[] infGroups=consInfShow.getInformantGroups();
		WSPersonBean[] infPersons=consInfShow.getInformantPersons();
		System.out.println("Informants");
		if(infGroups==null||infGroups.length==0){
			System.out.println("\tNo groups:");
		}else{
			System.out.println("\tGroups:");
			for (int i = 0; i < infGroups.length; i++) {
				System.out.println("\t\t"+infGroups[i].getPersonID()+":"+infGroups[i].getFullName());
			}
		}
		if(infPersons==null||infPersons.length==0){
			System.out.println("\tNo persons:");
		}else{
			System.out.println("\tPersons:");
			for (int i = 0; i < infPersons.length; i++) {
				System.out.println("\t\t"+infPersons[i].getPersonID()+":"+infPersons[i].getFullName());
			}
		}
	}
	
	
	private void printAttachments(WSAttachmentBean[] attchments) {
		if(attchments==null||attchments.length==0){
			System.out.println("No attachments");
		}else{
			System.out.println("Attachemnts: "+attchments.length);
			for (int i = 0; i < attchments.length; i++) {
				System.out.println("\t"+attchments[i].getAttachKey()+":"+attchments[i].getAttachName()+" "+attchments[i].getSizeKb()+"Kb"+" By "+attchments[i].getOriginator().getFullName());
			}
		}
	}
	
	private void printComments(WSCommentEdit[] comments) {
		if(comments==null||comments.length==0){
			System.out.println("No comments");
		}else{
			System.out.println("Comments: "+comments.length);
			for (int i = 0; i < comments.length; i++) {
				System.out.println("\t By "+comments[i].getOriginator().getFullName()+" At"+comments[i].getCreatedDate());
				System.out.println(comments[i].getDescription()+"\n");
			}
			System.out.println();
		}
	}
	
	private void printComments(WSComment[] comments) {
		if(comments==null||comments.length==0){
			System.out.println("No comments");
		}else{
			System.out.println("Comments: "+comments.length);
			for (int i = 0; i < comments.length; i++) {
				System.out.println("\t By "+comments[i].getOriginator().getFullName()+" At"+comments[i].getCreatedDate());
				System.out.println(comments[i].getDescription()+"\n");
			}
			System.out.println();
		}
	}
	
	public static void main(String[] args) {
		/*int NumberOfThreads = 10;
		int NumberInParallel = 2;
		
		for (int i = 0; i < Math.ceil(NumberOfThreads/NumberInParallel); ++ i) {
			for (int j = 0; j < NumberInParallel; ++j) {
				(new Thread(new ItemSample())).start();
			}
		}*/
		ItemSample itemSample=new ItemSample();
		itemSample.run();
	}
	
	public void run() {
		this.login(uName,pField);
		long start = new Date().getTime();
		//String itemID=this.createItem();
		String itemID="3";

		if(itemID!=null){
			//this.getEditItemContext(itemID);
			//this.getItem(itemID);
			// this.getDisplayItem(itemID);
			//this.getItemContext(itemID);
			/*this.getItemContexts(new String[]{itemID});
			this.getEditItemContext(itemID);
			this.modifyItem(itemID);
			this.addComment(itemID);
			this.editComment(itemID);
			this.deleteComments(itemID);
			this.changeStatus(itemID);
			this.getHistory(itemID);
			this.archiveItem(itemID);
			this.unarchiveItem(itemID);
			this.deleteItem(itemID);
			this.undeleteItem(itemID);*/
		}

		int icount = 0;
//		for (int i = 1; i <= 1800; ++ i) {
//			this.createItem("Web service item " + String.valueOf(i), i-1);
//			++icount;
//		}
		this.createItems(500000,null);
		
	}
}
