/*******************************************************************************
 * Copyright (C) 2015 Steinbeis GmbH & Co. KG.
 * All rights reserved. 
 *
 * This program and the accompanying materials are made 
 * available under the terms of the Track+ License v2.0
 * which is available at http://www.trackplus.com/tpl-v20.html
 *
 * $Id:$
 *******************************************************************************/
package com.trackplus.track.ws.samples;

import java.rmi.RemoteException;

import com.trackplus.track.ws.beans.WSListBean;
import com.trackplus.track.ws.beans.WSListOptionBean;
import com.trackplus.track.ws.tcl.TCLFacade;
import com.trackplus.track.ws.tcl.TCLFacadeException;

/**
 * This class provides handling for custom selection lists (tags, labels). You
 * can add, remove, and modify lists.
 * 
 */
public class ListSample extends LoginSample {

	/**
	 * Constructor
	 */
	public ListSample() {
		super();
	}

	public ListSample(TCLFacade connector) {
		super(connector);
	}

	/**
	 * Get all lists available to you.
	 * 
	 */
	public WSListBean[] getGlobalLists() {
		try {
			WSListBean[] lists = connector.getGlobalLists();
			if (lists != null) {
				WSListBean list;
				for (int i = 0; i < lists.length; i++) {
					list = lists[i];
					System.out.println(i + " {name=" + list.getName() + "; objectID=" + list.getObjectID() + "}");
				}
			} else {
				System.out.println("No global list");
			}
			return lists;
		} catch (RemoteException e) {
			e.printStackTrace();
			return null;
		} catch (TCLFacadeException e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Add a selection list. Later you can ad options to this list.
	 * 
	 */
	public String addList() {
		WSListBean list = new WSListBean();
		list.setName("myListFromWebservice");
		list.setDescription("list created from web-service");
		try {
			return connector.saveList(list);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Retrieve all options for a given list.
	 * 
	 * @param listID
	 */
	public WSListOptionBean[] getOptionsByList(String listID) {
		try {
			WSListOptionBean[] options = connector.getListOptions(listID);
			return options;
		} catch (RemoteException e) {
			e.printStackTrace();
			return null;
		} catch (TCLFacadeException e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Delete an option by its object id.
	 * 
	 * @param optionID
	 */
	public void deleteOption(String optionID) {
		try {
			connector.deletListOption(optionID);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			e.printStackTrace();
		}
	}

	private void printOptions(WSListOptionBean[] options) {
		if (options != null) {
			WSListOptionBean option;
			for (int i = 0; i < options.length; i++) {
				option = options[i];
				System.out.println(i + " {name=" + option.getLabel() + "; objectID=" + option.getObjectID() + "}");
			}
		} else {
			System.out.println("null options");
		}
	}

	/**
	 * Add an option to a given list.
	 * 
	 * @param listID
	 *            the object id of the list this option should be added to
	 * @param label
	 *            the label for this option
	 */
	public String addListOption(String listID, String label) {
		WSListOptionBean option = new WSListOptionBean();
		option.setListID(listID);
		option.setLabel(label);
		try {
			return connector.saveListOption(option);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			e.printStackTrace();
		}
		return null;
	}

	public static void main(String[] args) {

		if (args.length >= 2) {
			uName = (args[0]);
			pField = args[1];
		}
		if (args.length > 2) {
			url = args[3];
		}
		ListSample listSample = new ListSample();
		// login
		listSample.login(uName, pField);

		String listID = listSample.addList();
		String option1ID = null;

		if (listID != null) {
			option1ID = listSample.addListOption(listID, "option1");
			listSample.addListOption(listID, "option2");
		}
		WSListBean[] globalLists = listSample.getGlobalLists();
		if (globalLists != null && globalLists.length > 0) {
			System.out.println("\n\n");
			WSListBean listBean;
			for (int i = 0; i < globalLists.length; i++) {
				listBean = globalLists[i];
				listID = listBean.getObjectID();
				WSListOptionBean[] options = listSample.getOptionsByList(listID);
				System.out.println("List " + listID + ": " + listBean.getName());
				System.out.println("Options:");
				listSample.printOptions(options);
				System.out.println("\n");
			}

			if (option1ID != null) {
				listSample.deleteOption(option1ID);
				WSListOptionBean[] options = listSample.getOptionsByList(listID);
				System.out.println("Options after delete:");
				listSample.printOptions(options);
			}
		}
	}
}
