/*
 * This file is part of the Track+ application, a
 * tool for project and change management.
 *
 * Copyright (C) 2008 Trackplus
 *
 * Use and distribution of this software is governed by the
 * Track+ license conditions.
 *
 * $Id: TCLFacade.java,v 1.27 2008-03-28 14:44:47 adib Exp $
 */

package com.trackplus.track.ws.samples;

import java.rmi.RemoteException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.Locale;

import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509TrustManager;

import com.trackplus.track.ws.beans.WSLogonBean;
import com.trackplus.track.ws.beans.WSPersonBean;
import com.trackplus.track.ws.tcl.TCLFacade;
import com.trackplus.track.ws.tcl.TCLFacadeException;

/**
 * <p>
 * This class demonstrates how to use the TCL (Track+ Client Library) to login into a Track+ server
 * via the web service interface. You need to replace the URL below and have it point
 * to your server. Furthermore you need to set the user login name and
 * password.</p>
 * <p>
 * This example is a Java console application. You can run it directly by
 * calling its <code>main()</code> method.</p>
 *
 */
public class LoginSample implements Runnable {
	/**
	 * The URL to the Track+ web service. It is of the form
	 * <code>"http://yourdomain.com/track/services/TrackplusService"</code>
	 */
	 protected static String url = "http://localhost:8080/track";
	// protected static String url = "https://cloud.trackplus.com/dungs";

	// TC.login("admin","test276","https://www.trackplus.com/track");
	//protected static String url = "http://localhost:8080/track"; //"https://www.trackplus.com/track";
	// protected static String url = "https://wwwitrt.hs-esslingen.de/track"; //"https://www.trackplus.com/track";


	 /**
	 * The user login name
	 */
	protected static String uName = "admin";//"guest";

	/**
	 * The user password
	 */
//	protected static String pField ="Marc276%!";// "trackplus";
	protected static String pField ="tissi";// "trackplus";

	/**
	 * The client facade for the web service. This serves to establish the connection
	 * to the server, and all requests to the server are handled via this interface.
	 */
	protected TCLFacade connector;

	private long start;

	/**
	 * Constructor
	 */
	public LoginSample(){
//		TrustManagerFactory trustManagerFactory;
//		try {
//			trustManagerFactory = TrustManagerFactory.getInstance("PKIX");
//
//			trustManagerFactory.init((KeyStore) null);
//			TrustManager[] trustManagers =
//			    trustManagerFactory.getTrustManagers();
//			X509TrustManager trustManager =
//			    (X509TrustManager) trustManagers[0];
//			X509Certificate[] acceptedIssuers =
//			    trustManager.getAcceptedIssuers();
//		} catch (NoSuchAlgorithmException | KeyStoreException e1) {
//			// TODO Auto-generated catch block
//			e1.printStackTrace();
//		}
		try {
			connector = new TCLFacade(url);
		} catch (RemoteException e) {
			// url is invalid
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			//  url is invalid
			e.printStackTrace();
		}
	}

	public LoginSample(TCLFacade connector){
		this.connector=connector;
	}

	public static void main(String[] args) {
		
		System.err.println(System.getProperty("javax.net.ssl.trustStore"));

		//System.setProperty("javax.net.ssl.trustStore","/opt/trackplus/keystore/trackplus.com.ks");
		//System.setProperty("javax.net.ssl.trustStorePassword","marc276");
		System.setProperty("javax.net.debug","ssl");

		/*
		if (args.length < 2) {
			System.err.println("Usage: LoginSample <user> <password> [<Track+ server url>]");
			System.err.println("Example: LoginSample admin tissi http://localhost:8080/track");
			System.err.println();
			return;
		}
		*/

		LoginSample loginSample=new LoginSample();

		loginSample.login(uName, pField);

		//Incorrect login
		loginSample.login(uName, null);

		//Check version
		loginSample.checkVersion();
	}


	public void login(){
		login(uName,pField);
	}

	/**
	 * Login via web-service in Track+ system
	 * @param userlogin
	 * @param password
	 */
	public void login(String userlogin,String password){
		System.out.println("\n> LoginSample.login()");
		//create login information
		WSLogonBean logonBean=new WSLogonBean();
		logonBean.setUsername(userlogin);
		logonBean.setPassword(password);

		/*
		 * Setting the client locale
		 * If on the server there is no preferred locale for this user
		 * we use this locale.
		*/
		Locale clientLocale = Locale.getDefault();
		logonBean.setLocale(clientLocale.toString());
		try {
			WSPersonBean personBean= connector.login(logonBean);
			System.out.println("  < Login successful as: "+personBean.getFullName());
			System.out.println("  < Person preferred locale: "+personBean.getPreferredLocale());
		}
		catch (RemoteException e) {
			//connection problems like server is not started, connection time-out,...
			System.out.println("  < RemoteException.message: "+e.getMessage());
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		System.out.println("< LoginSample.login()\n");
	}

	/**
	 * Check the server version and print the compatible versions
	 */
	public void checkVersion(){
		System.out.println("\n> LoginSample.checkVersion()");
		try {
			boolean b=connector.checkVersion();
			if(b){
				System.out.println("  < Client and server versions are compatible!");
			}else{
				System.out.println("  < INCOMPATIBLE client and server versions!!!");
			}
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}

		// getting the server compatible versions

		// the first element in the array is the server version
		// and the rest are the client compatible versions
		try {
			String[] serverVersionAndCompatibility=connector.getServerVersionAndCompatible();
			System.out.println("  < Server version: "+serverVersionAndCompatibility[0]);
			System.out.print("  < Client compatible versions are: ");
			for (int i = 1; i < serverVersionAndCompatibility.length; i++) {
				System.out.print(serverVersionAndCompatibility[i]+";");
			}
			System.out.println();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		System.out.println("< LoginSample.checkVersion()\n");
	}

	/**
	 * Simple Handler for  TCLFacadeException
	 * @param e
	 */
	protected void handleTCLFacadeException(TCLFacadeException e) {
		//handling the messages
		System.out.println("TCLFacadeException.getErrorCode()="+e.getErrorCode());
		//these messages are localized from server
		String[] errorMessages=e.getErrorMessages();
		if(errorMessages!=null&&errorMessages.length>0){
			for (int i = 0; i < errorMessages.length; i++) {
				System.out.println("TCLFacadeException.ErrorMessage["+i+"]="+errorMessages[i]);
			}
		}
		System.out.println("  < TCLFacadeException.message:"+e.getMessage());
	}

	public void run() {

	}

	public long startTime() {
		start = new Date().getTime();
		return start;
	}

	public void stopTime(String msg) {
		System.err.println(msg + " (" + (new Date().getTime()-start) + " ms)");
		start = new Date().getTime();
	}

}
