/*
 * This file is part of the Track+ application, a
 * tool for project and change management.
 *
 * Copyright (C) 2008 Trackplus
 *
 * Use and distribution of this software is governed by the
 * Track+ license conditions.
 *
 * $Id$
 */
package com.trackplus.track.ws.samples;

import java.rmi.RemoteException;

import com.trackplus.track.ws.beans.WSItemBean;
import com.trackplus.track.ws.beans.WSPersonBean;
import com.trackplus.track.ws.beans.WSPersonEditBean;
import com.trackplus.track.ws.beans.WSPersonEditWithContext;
import com.trackplus.track.ws.tcl.TCLFacade;
import com.trackplus.track.ws.tcl.TCLFacadeException;

public class PersonSample extends LoginSample{
	public PersonSample() {
		super();
	}

	public PersonSample(TCLFacade connector) {
		super(connector);
	}
	
	/**
	 * Return all users in the database in an array. This is for view only, not for editing
	 * a user entry.
	 * @return all users in the datbase
	 */
	public WSPersonBean[] getAllUsers(){
		try {
			WSPersonBean[] allPersons=connector.getAllPersons();
			printPersons(allPersons);
			return allPersons;
		} catch (RemoteException e) {
			e.printStackTrace();
			return null;
		} catch (TCLFacadeException e) {
			e.printStackTrace();
			return null;
		}
	}

	public void printPersons(WSPersonBean[] allPersons) {
		if(allPersons!=null){
			WSPersonBean person;
			for(int i=0;i<allPersons.length;i++){
				person=allPersons[i];
				System.out.println(i+" {Name="+person.getFullName()+" ;  personID="+person.getPersonID()+"; loginName="+person.getLoginName()+"}");
			}
		}
	}
	
	/**
	 * Get a bean of a user based on his object id. The result is suitable for
	 * changing the users properties and saving back into the database.
	 * @param personID the object id of the user
	 * @return a bean with all user properties
	 */
	public WSPersonEditBean getPersonEdit(String personID){
		try {
			WSPersonEditBean personEditBean=connector.getEditPerson(personID);
			if(personEditBean!=null){
				printEditPerson(personEditBean);
			}
			return personEditBean;
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return null;
	}
	
	/**
	 * Get a bean of a user based on his login name. The result is suitable for
	 * changing the users properties and saving back into the database.
	 * @param loginName the login name of the user
	 * @return a bean with all user properties
	 */
	public WSPersonEditBean getPersonEditByLoginName(String loginName){
		System.out.println("/nLoading person by loginName:"+loginName);
		try {
			WSPersonEditBean personEditBean=connector.getEditPersonByLoginName(loginName);
			if(personEditBean!=null){
				printEditPerson(personEditBean);
			}
			return personEditBean;
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return null;
	}
	
	/**
	 * Delete a user from the database based on its object id.
	 * @param personID the object id of the user to be deleted.
	 */
	public void deletePerson(String personID){
		try{
			connector.deletePerson(personID);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/*
	 * Just a helper method showing the user bean properties available.
	 */
	private void printEditPersonContext(WSPersonEditWithContext personEditContext ){
		System.out.println("locales="+personEditContext.getLocales());
		System.out.println("timeZones="+personEditContext.getTimeZones());
		System.out.println("remindPriorityLevels="+personEditContext.getRemindPriorityLevels());
		System.out.println("remindSeverityLevels="+personEditContext.getRemindSeverityLevels());
		System.out.println("remindMeOnDaysList="+personEditContext.getRemindMeOnDaysList());
		System.out.println("departmentTree="+personEditContext.getDepartmentTree());
		System.out.println("designPaths="+personEditContext.getDesignPaths());
		System.out.println("substitutePersonsList="+personEditContext.getSubstitutePersonsList());
	}

	private void printEditPerson(WSPersonEditBean personEditBean) {
		System.out.println("\nWSPersonEditBean:[");
		System.out.println("objectID="+personEditBean.getObjectID());
		System.out.println("userName="+personEditBean.getUserName());
		System.out.println("firstName="+personEditBean.getFirstName());
		System.out.println("lastName="+personEditBean.getLastName());
		System.out.println("userEmail="+personEditBean.getUserEmail());
		// System.out.println("ldapOn="+personEditBean.getLdapOn());
		// System.out.println("forceLdap="+personEditBean.getForceLdap());
		// System.out.println("ldapUser="+personEditBean.getLdapUser());
		System.out.println("locale="+personEditBean.getLocale());
		
		
		// System.out.println("adminOrGuest="+personEditBean.getAdminOrGuest());
		System.out.println("domainPat="+personEditBean.getDomainPat());
		System.out.println("userTz="+personEditBean.getUserTz());
		
		
		// System.out.println("noEmail="+personEditBean.getNoEmail());
		System.out.println("emailLead="+personEditBean.getEmailLead());
//		System.out.println("remindMeAsOriginator="+personEditBean.getRemindMeAsOriginator());
//		System.out.println("remindMeAsManager="+personEditBean.getRemindMeAsManager());
//		System.out.println("remindMeAsResponsible="+personEditBean.getRemindMeAsResponsible());
		
		System.out.println("remindPriorityLevel="+personEditBean.getRemindPriorityLevel());
		System.out.println("remindSeverityLevel="+personEditBean.getRemindSeverityLevel());
		System.out.println("prefEmailType="+personEditBean.getPrefEmailType());
		System.out.println("remindMeOnDays="+personEditBean.getRemindMeOnDays());
		
		
		
		System.out.println("phone="+personEditBean.getPhone());
		
		System.out.println("department="+personEditBean.getDepartment());
		System.out.println("workingHours="+personEditBean.getWorkingHours());
		System.out.println("employeeId="+personEditBean.getEmployeeId());
		System.out.println("lastLogin="+personEditBean.getLastLogin());
		System.out.println("csvSeparator="+personEditBean.getCsvSeparator());
		System.out.println("csvEncoding="+personEditBean.getCsvEncoding());
		// System.out.println("saveAttachments="+personEditBean.getSaveAttachments());
		System.out.println("sessionTimeout="+personEditBean.getSessionTimeout());
		
		System.out.println("designPath="+personEditBean.getDesignPath());
		// System.out.println("activateInlineEdit="+personEditBean.getActivateInlineEdit());
		// System.out.println("activateLayout="+personEditBean.getActivateLayout());
		System.out.println("homePage="+personEditBean.getHomePage());
		
		System.out.println("substituteID="+personEditBean.getSubstituteID());
		
		System.out.println("]\n");
		
	}
	public String createUser(int index){
		WSPersonEditBean personEditBean=new WSPersonEditBean();
		personEditBean.setFirstName("firstName"+index);
		personEditBean.setLastName("lastName"+index);
		personEditBean.setUserEmail("user"+index+"@trackplus.com");
		personEditBean.setUserName("user"+index);
		personEditBean.setPasswd("user"+index);
		personEditBean.setPasswd2("user"+index);
		String personID=null;
		try {
			personID=connector.savePerson(personEditBean);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return personID;
	}
	private void replaceAndDeletePerson(){
		ItemSample itemSample=new ItemSample(connector);
		String personID1=createUser(1);
		if(personID1!=null){
			System.out.println("User created:"+personID1);
			//create an item and  set user1 as responsible
			String itemID=itemSample.createItem();
			if(itemID!=null){
				System.out.println("Item created:"+itemID);
				WSItemBean itemBean=null;
				try {
					itemBean = connector.getItem(itemID);
				} catch (RemoteException e1) {
					e1.printStackTrace();
				} catch (TCLFacadeException e1) {
					e1.printStackTrace();
				}
				if(itemBean!=null){
					itemBean.setResponsibleID(personID1);
					try {
						connector.saveItem(itemBean);
					} catch (RemoteException e) {
						e.printStackTrace();
					} catch (TCLFacadeException e) {
						e.printStackTrace();
					}
				}
			}
			
			//try to delete user
			try {
				connector.deletePerson(personID1);
				System.out.println("Person was simple deleted!");
			} catch (RemoteException e) {
				e.printStackTrace();
			} catch (TCLFacadeException e) {
				if(e.getErrorCode()==TCLFacadeException.DELETE_ERROR_CODES.NEED_REPLACE){
					System.out.println("The user can't be deleted, there is need for replacement.");
					String personID2=createUser(2);
					System.out.println("Create new user:"+personID2);
					//replace and delete person
					try {
						connector.replaceAndDeletePerson(personID1, personID2);
						System.out.println("Person succesfully replaced");
					} catch (RemoteException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					} catch (TCLFacadeException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
					
				}else{
					e.printStackTrace();
				}
			}
			
		}
	}

	/**
	 * Demonstrator for how to add, view, modify, and delete users.
	 * @param args
	 */
	public static void main(String[] args) {
		PersonSample personSample=new PersonSample();
		//login
		personSample.login(uName,pField);
		
		WSPersonBean[]  allPersons=personSample.getAllUsers();
		personSample.printPersons(allPersons);
		if(allPersons!=null&&allPersons.length>0){
			String personID=allPersons[1].getPersonID();
			String loginName=allPersons[1].getLoginName();
			
			personSample.getPersonEditByLoginName(loginName);
			
			/*WSPersonEditBean personEditBean=personSample.getPersonEdit(personID);
			personEditBean.setObjectID(null);
			personEditBean.setUserName("myUserName");
			try {
				personID=personSample.connector.savePerson(personEditBean);
				System.out.println("After add:");
				personSample.getAllUsers();
				personSample.deletePerson(personID);
				System.out.println("After delete:");
				personSample.getAllUsers();
			} catch (RemoteException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (TCLFacadeException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}*/	
		}
		personSample.replaceAndDeletePerson();
		
		
	}


}
