/*
 * This file is part of the Track+ application, a
 * tool for project and change management.
 *
 * Copyright (C) 2008 Trackplus
 *
 * Use and distribution of this software is governed by the
 * Track+ license conditions.
 *
 * $Id$
 */
package com.trackplus.track.ws.samples;

import java.rmi.RemoteException;

import com.trackplus.track.ws.beans.WSLabelValueBean;
import com.trackplus.track.ws.beans.WSProjectEditBean;
import com.trackplus.track.ws.beans.WSProjectEditWithContext;
import com.trackplus.track.ws.beans.WSTreeNode;
import com.trackplus.track.ws.tcl.TCLFacade;
import com.trackplus.track.ws.tcl.TCLFacadeException;

public class ProjectSample extends LoginSample{
	public ProjectSample() {
		super();
	}

	public ProjectSample(TCLFacade connector) {
		super(connector);
	}

	public WSLabelValueBean[] getAllProjects(){
		WSLabelValueBean[] projects=null;
		try {
			projects=connector.getAllProjects();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		return projects;
	}

	public WSLabelValueBean[] getUsedProjects(){
		WSLabelValueBean[] projects=null;
		try {
			projects=connector.getUsedProjects();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		return projects;
	}

	public WSLabelValueBean[] getProjects(){
		WSLabelValueBean[] projects=null;
		try {
			projects=connector.getProjects();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		return projects;
	}

	public WSTreeNode[] getProjectsAsTree(){
		// System.out.println("\n->ProjectSample.getProjectsAsTree()");
		WSTreeNode[] projects=null;
		try {
			projects=connector.getProjectsAsTree();
			if(projects==null||projects.length==0){
				System.out.println("No projects found on the server. Maybe the user does not have right in any project");
			}else{
				// System.out.println("Successfully obtained "+projects.length+ " projects:");
				for (int i = 0; i < projects.length; i++) {
					// BaseOptionSample.printNode(projects[i],"");
				}
				// System.out.println();
			}
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		// System.out.println("<-ProjectSample.getProjectsAsTree()\n");
		return projects;
	}


	public String addProject(){
		return addProject("");
	}

	public String addProject(String suffix){
		try {
			WSProjectEditWithContext projectEditContextBean=connector.getNewProjectWithContext();
			WSProjectEditBean projectEditBean=new WSProjectEditBean();

			WSLabelValueBean[] projectTypes= projectEditContextBean.getProjectTypeList();
			WSLabelValueBean[] projectStateList=projectEditContextBean.getProjectStateList();

			String projectName="my project"+suffix;
			projectEditBean.setName(projectName);
			String projectTypeID=projectTypes[0].getValue();
			projectEditBean.setProjectType(projectTypeID);
			String projectStateID=projectStateList[0].getValue();
			projectEditBean.setProjectState(projectStateID);

			String projectID=connector.saveProject(projectEditBean);
			return projectID;
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			e.printStackTrace();
		}
		return null;
	}

	public WSProjectEditBean getProjectEdit(String projectID){
		try {
			WSProjectEditBean projectEditBean=connector.getEditProject(projectID);
			if(projectEditBean!=null){
				printEditProject(projectEditBean);
			}
			return projectEditBean;
		} catch (RemoteException re) {
			re.printStackTrace();
		} catch (TCLFacadeException te) {
			System.err.println(te.getMessage());
			te.printStackTrace();
		}

		return null;
	}


	public void deleteProject(String projectID){
		try{
			connector.deleteProject(projectID);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void printAllProject(WSLabelValueBean[] allProjects) {
		if(allProjects!=null){
			WSLabelValueBean bean;
			for(int i=0;i<allProjects.length;i++){
				bean=allProjects[i];
				System.out.println(i+" {name="+bean.getLabel()+" ;ID="+bean.getValue()+"}");
			}
		}
	}

	private void printEditProject(WSProjectEditBean projectEditBean) {
		System.out.println("\nWSProjectEditBean:[");
		System.out.println("objectID="+projectEditBean.getObjectID());



		System.out.println("]\n");

	}

	public static void main(String[] args) {
		
		System.setProperty("javax.net.ssl.trustStore", "C:\\Program Files\\Java\\jre1.8.0_131\\lib\\security\\cacerts");
		System.setProperty("javax.net.ssl.trustStorePassword", "changeit");
		
		System.setProperty("javax.net.ssl.keyStore", "C:\\Program Files\\Java\\jre1.8.0_131\\lib\\security\\cacerts");
		System.setProperty("javax.net.ssl.keyStorePassword", "changeit");
//		System.out.println(System.setProperty("javax.net.ssl.trustStorePassword", "changeit"));
		
		ProjectSample projectSample=new ProjectSample();
		//login
		projectSample.login(uName,pField);
/*		WSTreeNode[]  projectsTree=projectSample.getProjectsAsTree();
		if(projectsTree!=null){
			String projectID=projectsTree[0].getId();
			WSProjectEditBean projectEditBean=projectSample.getProjectEdit(projectID);
		}

		projectSample.addProject();*/
		WSLabelValueBean[] allProjects=projectSample.getAllProjects();
		projectSample.printAllProject(allProjects);
		
		projectSample.getProjectsAsTree();

/*
		allProjects=projectSample.getUsedProjects();
		projectSample.printAllProject(allProjects);

		allProjects=projectSample.getProjects();
		projectSample.printAllProject(allProjects);
*/

	}

}
