/*
 * This file is part of the Track+ application, a
 * tool for project and change management.
 *
 * Copyright (C) 2008 Trackplus
 *
 * Use and distribution of this software is governed by the
 * Track+ license conditions.
 *
 * $Id: TCLFacade.java,v 1.27 2008-03-28 14:44:47 adib Exp $
 */

package com.trackplus.track.ws.samples;

import java.rmi.RemoteException;
import java.util.Date;

import com.trackplus.track.ws.beans.WSQueryConfigBean;
import com.trackplus.track.ws.beans.WSQueryResultItemBean;
import com.trackplus.track.ws.beans.WSTreeNode;
import com.trackplus.track.ws.tcl.TCLFacade;
import com.trackplus.track.ws.tcl.TCLFacadeException;

/**
 * <p>
 * This class demonstrates how to retrieve a number of issues from a Track+
 * server by a query via the web service interface using the Track+ Client
 * Library (TCL). The class uses the {@link LoginSample} class to perform
 * the login. You need to set the URL and the user name and password in that
 * class before you compile and run this class here.</p>
 * <p>
 * This example is a Java console application. You can run it directly by
 * calling its <code>main()</code> method.</p>
 *
 */
public class QuerySample extends LoginSample{
	public QuerySample() {
		super();
	}
	public QuerySample(TCLFacade connector) {
		super(connector);
	}
	public WSTreeNode[] getQueries() {
		System.out.println("\n->QuerySample.getQueries()");
		WSTreeNode[] result=null;
		try {
			result = connector.getQueries();
			printQueries(result);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.out.println("<-QuerySample.getQueries()\n");
		return result;
	}
	private void printQueries(WSTreeNode[] nodes){
		if(nodes!=null){
			for (int i = 0; i < nodes.length; i++) {
				BaseOptionSample.printNode(nodes[i],"");
			}
		}
	}

	public void executeQuery(String queryID) {
		try {
			WSQueryResultItemBean[] reportBeans=connector.executeQuery(queryID);
			printReportBeans(reportBeans);
			
		}catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
	}

	public void executeCustomQuery() {
		System.out.println("\n->QuerySample.executeCustomQuery()");
		WSQueryConfigBean reportConfigBean=new WSQueryConfigBean();
		try {
			reportConfigBean=connector.loadQueryConfigBean(reportConfigBean, true);
			//set states 1 (opened);
			reportConfigBean.setSelectedStates(new String[]{"1"});
		} catch (RemoteException e1) {
			e1.printStackTrace();
		} catch (TCLFacadeException e1) {
			handleTCLFacadeException(e1);
		}
		try {
			WSQueryResultItemBean[] reportBeans=connector.executeCustomQuery(reportConfigBean);
			printReportBeans(reportBeans);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		System.out.println("<-QuerySample.executeCustomQuery()\n");
	}

	private void printReportBeans(WSQueryResultItemBean[] reportBeans) {
		if(reportBeans==null||reportBeans.length==0){
			System.out.println("No issues found.");
		}else{
			int printRows=reportBeans.length;
			System.out.println(printRows+" issues founds!");
			if(printRows>10){
				printRows=10;
				System.out.println("The first 10 issues:");
			}
			System.out.println("ItemNo: Project: Subsystem: Title: Status: Responsible: LastModified");
			for (int i = 0; i < printRows; i++) {
				WSQueryResultItemBean row=reportBeans[i];
				System.out.print(row.getObjectID()+": ");
				System.out.print(row.getProject().getLabel()+": ");
				System.out.print(row.getSynopsis()+": ");
				System.out.print(row.getResponsible().getFullName()+": ");
				System.out.print(row.getLastEdit()+"\n");
			}
		}
	}
	public void loadReportConfigBean() {
		System.out.println("\n->QuerySample.loadReportConfigBean()");
		WSQueryConfigBean reportConfigBean=new WSQueryConfigBean();
		try {
			reportConfigBean=connector.loadQueryConfigBean(reportConfigBean, true);
			System.out.println("Projects:"+reportConfigBean.getProjects().length);
			System.out.println("Issue types:"+reportConfigBean.getIssueTypes().length);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			handleTCLFacadeException(e);
		}
		System.out.println("<-QuerySample.loadReportConfigBean()\n");
	}
	
	public static void main(String[] args) {
		QuerySample querySample=new QuerySample();
		querySample.login();
		
		// WSTreeNode[] queries=querySample.getQueries();
		//if(queries!=null){
			String queryID=null;
			
//			if(queryID!=null){
				// querySample.executeQuery(queryID);
				try {

					long start = new Date().getTime();

					querySample.connector.getEditItemContext("1000");
					long now = new Date().getTime();
					System.err.println ((now-start));
				} catch (RemoteException | TCLFacadeException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} //connector; //.getEditItemContext("1000");
			//}
		//}
		
		//querySample.loadReportConfigBean();
		
		//querySample.executeCustomQuery();
	}

}