/*******************************************************************************
 * Copyright (C) 2015 Steinbeis GmbH & Co. KG.
 * All rights reserved.
 *
 * This program and the accompanying materials are made
 * available under the terms of the Track+ License v2.0
 * which is available at http://www.trackplus.com/tpl-v20.html
 *
 * $Id:$
 *******************************************************************************/
package com.trackplus.track.ws.samples;

import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Map;

import com.trackplus.track.ws.beans.WSGroupBean;
import com.trackplus.track.ws.beans.WSLabelValueBean;
import com.trackplus.track.ws.beans.WSPersonBean;
import com.trackplus.track.ws.beans.WSRoleAssignmentBean;
import com.trackplus.track.ws.beans.WSRoleAssignmentEditBean;
import com.trackplus.track.ws.tcl.TCLFacade;
import com.trackplus.track.ws.tcl.TCLFacadeException;

public class RoleSample extends LoginSample {

	/**
	 * The constructor
	 */
	public RoleSample() {
		super();
	}

	public RoleSample(TCLFacade connector) {
		super(connector);
	}

	/**
	 * Get an array with all roles with label as role name and value as role id.
	 *
	 */
	public WSLabelValueBean[] getAllRoles() {
		try {
			WSLabelValueBean[] allRoles = connector.getAllRoles();
			return allRoles;
		} catch (RemoteException e) {
			e.printStackTrace();
			return null;
		} catch (TCLFacadeException e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Get a array with all role assignments for all users and groups for a
	 * given project or workspace.
	 *
	 * @param projectID
	 *            the object id for the workspace to get the role assignments
	 *            for
	 */
	public WSRoleAssignmentBean[] getAssignmentRolesByProject(String projectID) {
		try {
			WSRoleAssignmentBean[] roles = connector.getAssignmentRolesByProject(projectID);
			return roles;
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Add a role assignment.
	 *
	 * @param roleAssignmentEditBean
	 */
	public void addRoleAssignment(WSRoleAssignmentEditBean roleAssignmentEditBean) {
		try {
			connector.addRoleAssignment(roleAssignmentEditBean);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Remove a role assignment.
	 *
	 * @param roleAssignmentEditBean
	 */
	public void removeRoleAssignment(WSRoleAssignmentEditBean roleAssignmentEditBean) {
		try {
			connector.removeRoleAssignment(roleAssignmentEditBean);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (TCLFacadeException e) {
			e.printStackTrace();
		}
	}

	private Map<String, String> labelValueBeansAsMap(WSLabelValueBean[] beans) {
		Map<String, String> map = new HashMap<String, String>();
		if (beans != null) {
			WSLabelValueBean bean;
			for (int i = 0; i < beans.length; i++) {
				bean = beans[i];
				map.put(bean.getValue(), bean.getLabel());
			}
		}
		return map;
	}

	private Map<String, String> personBeansAsMap(WSPersonBean[] beans) {
		Map<String, String> map = new HashMap<String, String>();
		if (beans != null) {
			WSPersonBean bean;
			for (int i = 0; i < beans.length; i++) {
				bean = beans[i];
				map.put(bean.getPersonID(), bean.getFullName());
			}
		}
		return map;
	}

	private Map<String, String> groupBeansAsMap(WSGroupBean[] beans) {
		Map<String, String> map = new HashMap<String, String>();
		if (beans != null) {
			WSGroupBean bean;
			for (int i = 0; i < beans.length; i++) {
				bean = beans[i];
				map.put(bean.getObjectID(), bean.getName());
			}
		}
		return map;
	}

	private void printRoleAssignments(WSRoleAssignmentBean[] roleAssignmentBeans, WSLabelValueBean[] allRoles, WSPersonBean[] allPersons,
			WSGroupBean[] allGroups) {
		if (roleAssignmentBeans == null || roleAssignmentBeans.length == 0) {
			System.out.println("No role assignment");
			return;
		}
		WSRoleAssignmentBean role;
		String projectID;
		String roleID;
		String roleName;
		String[] personIDs;
		String personID;
		String personName;
		Map<String, String> roleMap = labelValueBeansAsMap(allRoles);
		Map<String, String> personMap = personBeansAsMap(allPersons);
		Map<String, String> groupMap = groupBeansAsMap(allGroups);

		for (int i = 0; i < roleAssignmentBeans.length; i++) {
			role = roleAssignmentBeans[i];
			projectID = role.getProjectID();
			roleID = role.getRoleID();
			roleName = roleMap.get(roleID);
			personIDs = role.getPersonIDs();
			System.out.println("  roleID=" + roleID + " roleName=" + roleName);
			System.out.print  ("  persons=[");
			for (int j = 0; j < personIDs.length; j++) {
				personID = personIDs[j];
				personName = personMap.get(personID);
				if (personName == null) {
					personName = "GROUP-" + groupMap.get(personID);
				}
				System.out.print("{" + personID + ": " + personName + "} ");
			}
			System.out.println("]\n");
		}
	}

	/*
	 * Print the existing roles.
	 */
	private void printRoles(WSLabelValueBean[] allRoles) {
		if (allRoles != null) {
			WSLabelValueBean role;
			for (int i = 0; i < allRoles.length; i++) {
				role = allRoles[i];
				System.out.println(i + " {name=" + role.getLabel() + "; ID=" + role.getValue() + "}");
			}
		} else {
			System.out.println("No roles");
		}
	}

	public static void main(String[] args) {
		RoleSample roleSample = new RoleSample();

		// Login, the connection details are in parent class LoginSample
		roleSample.login();
		WSLabelValueBean[] allRoles = roleSample.getAllRoles();
		System.out.println("All roles:");
		roleSample.printRoles(allRoles);

		GroupSample groupSample = new GroupSample(roleSample.connector);
		WSGroupBean[] allGroups = groupSample.getAllGroups();

		PersonSample personSample = new PersonSample(roleSample.connector);
		WSPersonBean[] allPersons = personSample.getAllUsers();

		String projectID = null;
		String projectName = null;

		ProjectSample projectSample = new ProjectSample(roleSample.connector);
		WSLabelValueBean[] allProjects = projectSample.getAllProjects();

		if (allProjects != null && allProjects.length > 0) {
			WSRoleAssignmentBean[] roleAssignmentBeans;
			int i = 0;
			// for(i=0;i<allProjects.length;i++){
			projectID = allProjects[i].getValue();
			projectName = allProjects[i].getLabel();
			System.out.println("\nThe following listings are for project with projectID=" + projectID + ", projectName=" + projectName);
			roleAssignmentBeans = roleSample.getAssignmentRolesByProject(projectID);
			System.out.println("\nRole assignments:");
			roleSample.printRoleAssignments(roleAssignmentBeans, allRoles, allPersons, allGroups);
			System.out.println("");
			// }
		}

		if (projectID != null && allRoles != null && allRoles.length > 0 && allPersons != null && allPersons.length > 0) {
			// add role assignment
			String roleID = allRoles[0].getValue();
			String personID = allPersons[0].getPersonID();
			String groupID = allGroups[0].getObjectID();
			WSRoleAssignmentEditBean wsRoleAssignmentEditBean = new WSRoleAssignmentEditBean();
			wsRoleAssignmentEditBean.setProjectID(projectID);
			wsRoleAssignmentEditBean.setRoleID(roleID);
			wsRoleAssignmentEditBean.setPersonIDs(new String[] { personID, groupID });

			roleSample.addRoleAssignment(wsRoleAssignmentEditBean);
			WSRoleAssignmentBean[] roleAssignmentBeans = roleSample.getAssignmentRolesByProject(projectID);
			System.out.println("Role assignments after add:");
			roleSample.printRoleAssignments(roleAssignmentBeans, allRoles, allPersons, allGroups);

			// remove role
			roleSample.removeRoleAssignment(wsRoleAssignmentEditBean);

		}
	}

}
